######################
# 2章
######################
# 変数
######################
julia> x = 3 
julia> x + 4
7

julia> ans #一つ前の実行された値はansに保存されます。
7

julia> y = x + 3
6

julia> y
6

julia> ans
6

julia> z = "Hello World!"
"Hello World!"

# 整数と浮動小数点
######################
julia> typeof(5)     
Int64    # 64ビットの整数型

julia> typeof(10.5)     
Float64    # 64ビットの浮動小数点型

julia> typeof(3 + 10im) 
Complex{Int64}    # 複素数型

julia> typeof(4 // 7)  # 有理数型
Rational{Int64}

# データがオーバーフローした場合の処理
######################
julia> x = typemax(Int64)
9223372036854775807

julia> x + 1 
-9223372036854775808 

julia> x + 1 == typemin(Int64)
true

# 任意精度計算
######################
julia> BigInt(typemax(Int64)) + 1
9223372036854775808

julia> BigFloat(2.0^100) / 7
1.810929428897470573566718864822857142857142857142857142857142857142857142857139e+29

# 二項演算子
######################
julia> 10 + 15      
25 # 足し算

julia> 87 - 21      
66 # 引き算

julia> 5 * 21     
105 # 掛け算

julia> 80 / 8
10.0 # 割り算

julia> 7 / 3 
2.3333333333333335 # 整数型同士の割り算は、浮動小数点数となる

julia> div(9, 2) 
4 # 整数レベルで割り算するには、 div を使う

julia> 7.05 ^ 2 
49.7025 # べき乗

julia> 13 % 10 
3 # 余り

julia> (29 + 70) * 21 
2079 # 括弧で演算の優先順位を制御する

# 複素数や無理数の計算も同様にできます。
julia> (1 + 2im)*(4 - 5im) 
14 + 3im

julia> (1 + 2im)/(4 - 5im) 
-0.14634146341463417 + 0.3170731707317074im

julia> (1 + 2im) + (4 - 5im) 
5 - 3im

julia> (-3 + 2im) - (1 - 2im) 
-4 + 4im

# 数学の方程式
######################
julia> x = 5  
5

julia> 4x^2 - 5x + 3 
78

# 以下のように指数関数を記述することもできます。
julia> x = 3 
3 
julia> 2^2x 
64

# 複素数でも同様に数学の数式のように記述できます。
julia> 5(1 - 1im) 
5 - 5im

# ビット演算子
######################
julia> ~134
-135 #ビット単位の否定

julia> 156 & 245 
148 #ビット単位の論理積

julia> 345 ⊻ 678 
1023 #排他的論理和

julia> xor(345, 678) 
1023 #排他的論理和

julia> a = 1000 
1000 

julia> a = a >>> 2 
250 #右シフト（論理シフト）

julia> a = a >>> 3 
31 #右シフト（論理シフト）

julia> a = a >> 3
3 #右シフト（算術シフト）

julia> a = a << 3 
24 #左シフト（論理＆算術シフト）

julia> a = a << 3 
192 #左シフト（論理＆算術シフト）

# インクリメント演算子
######################
julia> x = 1
1

julia> x += 2 
3

julia> x 
3

# 比較演算子
######################
julia> 2 == 2 
true 

julia> 2 == 4 
false

julia> 2 != 4 
true 

julia> 3 == 3.0 
true 

julia> 3 < 6 
true 

julia> 3.0 > 4
false

julia> 3 >= 2.0
true 

julia>-10 <= 10 
true 

julia>-10 <= -10 
true 

julia>-10 <= -20 
false 

julia>33 < -0.3 
false

#　連続した比較演算子
#　以下のように連続して比較演算子を並べることも可能です。

julia> 15 < 20 <= 20 < 33 == 33 > 20 >= 11 == 11 < 30 != 50

# 文字列
######################
julia> "Hello, world.\n" 
"Hello, world.\n" 

julia> ans # Juliaではひとつ前の返り値をansで表します。
"Hello, world.\n"

julia> print("Hello World") # 改行なし出力
Hello World

julia> println("Hello World") # 改行付き出力
Hello World

julia> println("Hello World ¥n") # \nは改行文字を表す
Hello World

# 文字列 
julia> str = "Hello World" 
"Hello World"

julia> println(str)
Hello World

julia> str[3:5]
"llo"

julia> str[1:1]
"H"

julia> str[2:2]
"e"
julia> str[1:5]
"Hello"

julia> str[3:10]
"llo, wor"

julia> str[3:end] 
"llo, world.!"

julia> str[3:20]
ERROR: BoundsError: attempt to access Stringat index [3:20]Stacktrace:[1] checkbounds at ./strings/basic.jl:194 [inlined][2] getindex(::String, ::UnitRange{Int64}) at ./strings/string.jl:248[3] top-level scope at REPL[15]:1

# Char型
julia> str[1]
'H': ASCII/Unicode U+0048 (category Lu: Letter, uppercase)

julia> str[2]
'e': ASCII/Unicode U+0065 (category Ll: Letter, lowercase)

julia> str[end]
'!': ASCII/Unicode U+0021 (category Po: Punctuation, other)
julia> str[end-1] 
'.': ASCII/Unicode U+002E (category Po: Punctuation, other)

# 文字列の連結
######################
julia> a = "Hello"
"Hello"

julia> b = "world"
"world"

julia> string(a, ", ", b, "!")
"Hello, world!"

julia> a * ", " * b * "!" 
"Hello, world!"

# 補間
julia> "$a, $b !"
"Hello, world !"

# 特殊文字の入力
######################
julia> π 
π = 3.1415926535897...



# 関数
######################
julia> function f(x,y)
          x + y
       end
f (generic function with 1 method)

julia> f(3,4)
7

julia> function h(x,y) 
           return x * y 
       end 
h (generic function with 1 method) 

julia> h(3,6) 
18

julia> f(x,y) = x + y
f (generic function with 1 method)

julia> ft(x::Int,y::Int) = x + y
f (generic function with 1 method)

julia> f(3,4.5)
7.5

julia> ft(3,4.5)
MethodError: no method matching ft(::Int64, ::Float64)

julia> ft(x::Int,y::Int)::Float64 = x + y
ft (generic function with 1 method)

julia> ft(5,6)
11.0

julia> f(x,y) = 3x + 2y + 1
f (generic function with 1 method)

julia> f(2,3)
13

julia> f(x) = 3x^2 + 2x + 1 
f (generic function with 2 methods)  

julia> f(2) 
17  

julia> g = f 
f (generic function with 2 methods)  

julia> g(2) 
17

julia> 4 + 5 + 6 
15  

julia> +(4, 5, 6) 
15  

julia> g = + 
+ (generic function with 184 methods)  

julia> g(4,5,6) 
15

julia> ∑(x,y) = x + y 
∑ (generic function with 1 method) 

julia> ∑(10, 20) 
30

julia> x -> 5x^2 + 4x + 12
#5 (generic function with 1 method)

julia> function (x)
           5x^2 + 4x + 12
       end
#7 (generic function with 1 method)

Julia> map(x -> 5x^2 + 4x + 12, [4, 3 ,-2])
3-element Array{Int64,1}:  
108   
69   
24

julia> function f(a, b)
           a+b, a*b
       end
f (generic function with 1 method)

julia> f(10, 20)
(30, 200)

julia> x, y = f(10, 20) 
(30, 200) 

julia> x 
30 

julia> y 
200

julia> bar(a,b, x...) = (a, b, x) 
bar (generic function with 1 method) 

julia> bar(1,2,3,4,5,6) 
(1, 2, (3, 4, 5, 6))

julia> f(x, y=10) = x * y;

# オプション引数
######################Julia> f(20)
200

Julia> f(20,30)
600

# キーワード引数
######################
julia> function plot(x, y; style="solid", width=1, color="black")     
           ### 
       end

# 制御フロー
######################
julia> z = begin 
       x = 10 
       y = 20 
       x * y 
       end 
200

julia> z = (x = 10; y = 20; x * y) 
200

julia> begin x = 10; y = 20; x * y end
200

julia> (x = 10;
       y = 20;
       x * y)
200


if x < y
    println("x is less than y")
elseif x > y
    println("x is greater than y")
else
    println("x is equal to y")
end

julia> function test(x, y)
       if x < y 
           println("x is less than y") 
       elseif x > y 
           println("x is greater than y") 
       else 
           println("x is equal to y") 
       end 
   end 
test (generic function with 1 method)

julia> test(100, 500) 
x is less than y 

julia> test(500, 100) 
x is greater than y 

julia> test(500, 500) 
x is equal to y

julia> x = 10; y = 20;
julia> println(x < y ? "less than" : "not less than")
less than

julia> x = 20; y = 10;
julia> println(x < y ? "less than" : "not less than")
not less than

# 短絡評価
######################
julia> i(x) = (println(x); true)
i (generic function with 1 method)

julia> h(x) = (println(x); false)
h (generic function with 2 methods)

julia> i(0) && i(1)
0
1
true

julia> i(0) && h(1)
0
1
false

julia> h(0) && h(1)
0
false

julia> h(0) && i(1)
0
false

julia> i(0) || i(1)
0
true

julia> i(0) || h(1)
0
true

julia> h(0) || i(1)
0
1
true

julia> h(0) || h(1)
0
1
false

julia> i = 0;
julia> while i <= 10
           println(i)
           global i += 2
       end
0
2
4
6
8
10

julia> for h = 1:7 
           println(h) 
       end 
1
2 
3 
4 
5 
6 
7

julia> try
           f = open("example.txt")
           lines = readlines(f)
           for line in lines
               println(line)
           end
           close(f)
       catch
           println("ファイルをオープンできません")
       finally
           println("処理が終了しました。")
       end
ファイルをオープンできません
処理が終了しました。


# JUliaのデータ型
######################
# 型の宣言
######################
julia> function f()
           x::Int64 = 10
           x
       end
f (generic function with 1 method)

julia> f()
10

julia> typeof(ans)
Int64


julia> function f(x)::Float64
       if x == 0
           return 1
       end
       return sin(x)/(pi*x)
    end
f (generic function with 2 methods)

julia> f(50)
-0.001670330196399726

# 抽象型
######################
abstract type Number end
abstract type Real     <: Number end
abstract type AbstractFloat <: Real end
abstract type Integer  <: Real end
abstract type Signed   <: Integer end
abstract type Unsigned <: Integer end

julia> Integer  <: Real
true

julia> Integer <: AbstractFloat
false

# プリミティブ型
######################
primitive type Float16 <: AbstractFloat 16 end
primitive type Float32 <: AbstractFloat 32 end
primitive type Float64 <: AbstractFloat 64 end

primitive type Bool <: Integer 8 end
primitive type Char <: AbstractChar 32 end

primitive type Int8    <: Signed   8 end
primitive type UInt8   <: Unsigned 8 end
primitive type Int16   <: Signed   16 end
primitive type UInt16  <: Unsigned 16 end
primitive type Int32   <: Signed   32 end
primitive type UInt32  <: Unsigned 32 end
primitive type Int64   <: Signed   64 end
primitive type UInt64  <: Unsigned 64 end
primitive type Int128  <: Signed   128 end
primitive type UInt128 <: Unsigned 128 end

# 複合型
######################
julia> struct CompT 
           avlue 
           bvlue::Int 
           cvlue::Float64 
       end

julia> f1 = CompT("Hello, world.", 39, 4.5)
CompT("Hello, world.", 39, 4.5)

julia> typeof(f1)
CompT

julia> f1.avlue
"Hello, world."

julia> f1.bvlue
39

julia> f1.cvlue
4.5

julia> mutable struct CompT2
           avalue
           bvlue::Int
           cvlue::Float64 
       end

julia> f1 = CompT2("Hello, world.", 34, 5.5)
CompT2("Hello, world.", 34, 5.5)

julia> f1.avlue = "Good day"
"Good day"

julia> f1.bvlue = 52
52

julia> f1
CompT2("Good day", 52, 5.5)

# Union型
######################
julia> IntOrString = Union{Int,AbstractString}
Union{Int64, AbstractString}

julia> 30 :: IntOrString 
30 

julia> "Good" :: IntOrString 
"Good" 

julia> 32.5 :: IntOrString 
ERROR: TypeError: in typeassert, expected Union{Int64, AbstractString}, got a value of type Float64

# パラメータ型
######################
julia> struct Point{T}
           x::T
           y::T
       end

julia> Point{Int64}
Point{Int64}

# パラメータ抽象型
######################
julia> abstract type Py{T} end

julia> Py{Int64} <: Py
true

julia> Py{21} <: Py
true

# データ型の演算
######################
# isa 関数は、オブジェクトが指定された型であるかどうかを検査し、真か偽を返します。
######################

julia> isa(10, Int)
true

julia> isa(100, AbstractFloat)
false

# typeof() 関数は、引数の型を返します。

julia> typeof(Rational{Int})
DataType

julia> typeof(Union{Real,Float64,Rational})
DataType

julia> typeof(Union{Real,String})
Union

# supertype() 関数は、型のスーパータイプを返します。

julia> supertype(Float64)
AbstractFloat

julia> supertype(Number)
Any

julia> supertype(AbstractString)
Any

julia> supertype(Any)
Any


# コレクション
######################
# タプル
######################
julia> t1 = ()
()

julia> println(isempty(t1))
true
julia> # Creating a tuple with similar values
julia> t2 = (1, 2, 4, 5)
(1, 2, 4, 5)

julia> println(t2)
(1, 2, 4, 5)   

julia> t2[3] 
4 

julia> t2[5] 
ERROR: BoundsError: attempt to access (1, 2, 4, 5) at index [5] 
Stacktrace: 
[1] getindex(::Tuple, ::Int64) at ./tuple.jl:24 
[2] top-level scope at REPL[10]:1


julia> # Creating a tuple with mixed values
julia> t3 = (1, 2, 3, "Hello World")
(1, 2, 3, "Hello World")

julia> println(t3)
(1, 2, 3, "Hello Geeks")

# 名前付きタプル
######################
# 例1：
julia> x = (a = 1, b = 2.0, c = "hello world")
(a = 1, b = 2.0, c = "hello world")

# 例2：
julia> mynames = (:a, :b, :c);
julia> myvalues = [1, 2.0, "hello world"];
julia> NamedTuple{mynames}(myvalues)
(a = 1, b = 2.0, c = "hello world")

# 例3：
(@1.5) pkg> add NamedTupleTools
julia> using NamedTupleTools
[ Info: Precompiling NamedTupleTools [d9ec5142-1e00-5aa0-9d6a-321866360f50]

julia> nt = namedtuple(Symbol.(["a", "b", "c"])...)

NamedTuple{(:a, :b, :c),T} where T<:Tuple
julia> nt.([1, 2.0, "hello world"]...)
(a = 1, b = 2.0, c = "hello world")

例４：
julia> mynames = (:a, :b, :c);
julia> myvalues = [1, 2.0, "hello world"];
julia> (;zip(mynames, myvalues)...)
(a = 1, b = 2.0, c = "hello world")


julia> x = (a = 1, b = 2.0, c = "hello world") 
(a = 1, b = 2.0, c = "hello world")  

julia> x.a 
1  

julia> x[2] 
2.0  

julia> x[:c] 
"hello world"

# keys関数、values関数を用いて、キーと値を取得できる。
julia> keys(x) 
(:a, :b, :c)

julia> values(x)
(1, 2.0, "hello world")

# 配列とその操作
######################
julia> a = Vector(1:7) 
7-element Array{Int64,1}:  
1  
2  
3  
4  
5  
6  
7  

# 単一要素の抽出
julia> a[1]
1

julia> a[begin]
1

julia> a[3]
3

julia> a[end]
5

# 複数要素の抽出
julia> a[3:end] 
5-element Array{Int64,1}:  
3  
4  
5  
6  
7  

julia> a[begin:end] 
7-element Array{Int64,1}:  
1  
2  
3  
4  
5  
6  
7  

julia> a[begin:5] 
5-element Array{Int64,1}:  
1  
2  
3  
4  
5 

julia> a[2:end-1] 
5-element Array{Int64,1}: 
2 
3 
4 
5 
6

julia> a[1:2:end]
4-element Array{Int64,1}:
1
3
5
7

# 関数による操作
######################
julia> A=[1, 2, 3, 4, 5, 6]
6-element Array{Int64,1}:
1
2
3
4
5
6

# 末尾の要素の取り出し
julia> pop!(A)
6

julia> A
5-element Array{Int64,1}:
1
2
3
4
5

# 末尾への要素の追加
julia> B = push!(A, 4, 5, 6);
julia> B
8-element Array{Int64,1}:
1
2
3
4
5
4
5
6

# 任意の部位への要素の追加
julia> insert!([6, 5, 4, 2, 1], 4, 3)
6-element Array{Int64,1}:
6
5
4
3
2
1

julia> foo = [4,2,1];
julia> insert!(foo, 2, 3);
julia> foo
4-element Array{Int64,1}:
4
3
2
1

# 任意の部位の要素の削除
julia> foo = [6, 5, 4, 3, 2, 1]
;julia> deleteat!(foo, 5)
;julia> foo
5-element Array{Int64,1}:
6
5
4
3
1

# 等差数列の作成
julia> Vector(1:7)  # start:end 
7-element Array{Int64,1}:  
1  
2  
3  
4  
5  
6  
7  

julia> Vector(1:2:7)  # start:step:end 
4-element Array{Int64,1}:  
1  
3  
5  
7  

julia> Array(1:2:7) 
4-element Array{Int64,1}:  
1  
3  
5  
7  

julia> [1:2:7;] 
4-element Array{Int64, 1}
1  
3  
5  
7  

julia> collect(1:2:7) 
4-element Array{Int64,1}:  
1  
3  
5  
7  

julia> [i for i in 1:2:7]  # 内包表記 4-element Array{Int64,1}:  
1  
3  
5  
7  


# 単位行列の作成
julia> using LinearAlgebra 
julia> Matrix(1.0I, 4, 4) 
4×4 Array{Float64,2}: 
1.0  0.0  0.0  0.0 
0.0  1.0  0.0  0.0 
0.0  0.0  1.0  0.0 
0.0  0.0  0.0  1.0 

julia> Matrix{Int32}(1I, (4, 4)) 
4×4 Array{Int32,2}: 
1  0  0  0 
0  1  0  0 
0  0  1  0 
0  0  0  1 

julia> Array(1I, (4, 4))  # Arrayの使用 
4×4 Array{Int64,2}: 
1  0  0  0 
0  1  0  0 
0  0  1  0 
0  0  0  1 

julia> collect(1I(4))  # collectの使用 
4×4 Array{Int64,2}: 
1  0  0  0 
0  1  0  0 
0  0  1  0 
0  0  0  1

# 二次元配列の作成と要素の抽出
julia> a = Vector(1:5); b = Vector(6:10); c = [a b]
5×2 Array{Int64,2}:
1   6
2   7
3   8
4   9
5  10

julia> c[1,2]
6

julia> c[begin,]
1

julia> c[end] 
10

julia> c[:, 1] #1列目
5-element Array{Int64,1}:
1
2
3
4
5

# Linear indexを要素にもつ配列による要素の抽出
julia> c[[2, 7]]  
2-element Array{Int64,1}:
2
7

# Cartesian indexを要素にもつ配列による要素の抽出
julia> c[[CartesianIndex(2, 1), CartesianIndex(2, 2)]]
2-element Array{Int64,1}:
2
7

# Cartesian indexとブロードキャスト.の使用による要素の抽出
julia> c[CartesianIndex.(2, 1:2)]
2-element Array{Int64,1}:
2
7

julia> d = reshape(1:16, (4, 4))
4×4 reshape(::UnitRange{Int64}, 4, 4) with eltype Int64:
1  5   9  13
2  6  10  14
3  7  11  15
4  8  12  16

julia> d[:, 3:end]
4×2 Array{Int64,2}:
9  13
10  14
11  15
12  16

julia> d[:, 1:1]
4×1 Array{Int64,2}:
1
2
3
4

julia> d[1:2, :]
2×4 Array{Int64,2}:
1  5   9  13
2  6  10  14

# 多次元配列Array型に関する操作
# 配列の初期化
# respahe の使用例
julia> A = Vector(1:9)
9-element Array{Int64,1}:
1
2
3
4
5
6
7

julia> reshape(A, (3, 3))
3×3 Array{Int64,2}:
1  4  7
2  5  8
3  6  9

julia> reshape(A, 3, :)
3×3 Array{Int64,2}:
1  4  7
2  5  8
3  6  9

julia> reshape(1:8, 2, 4)
2×4 reshape(::UnitRange{Int64}, 2, 4) with eltype Int64:
1  3  5  7
2  4  6  8

# 全要素が0の配列の作成
# Base.zerosの使用


julia> zeros(Int8, 2, 2)
2×2 Array{Int8,2}:
0  0
0  0

julia> zeros(Int8, (2, 2))
2×2 Array{Int8,2}:
0  0
0  0

julia> zeros((2, 2))
2×2 Array{Float64,2}:
0.0  0.0
0.0  0.0

# Base.zeroの使用
julia> a = rand(2, 2)
2×2 Array{Float64,2}:
0.32199   0.970492
0.979503  0.044438

julia> zero(a)
2×2 Array{Float64,2}:
0.0  0.0
0.0  0.0

# 全要素が1の配列
# Base.onesの使用
julia> ones(Int8, (3, 2, 1))
3×2×1 Array{Int8,3}:
[:, :, 1] =
1  1
1  1
1  1

# 任意の値で初期化した配列
# Base.fill
julia> fill(4, 3, 3) 
3×3 Array{Int64,2}: 
4  4  4 
4  4  4 
4  4  4 

julia> fill(2.0, (3, 3, 1)) 
3×3×1 Array{Float64,3}: 
[:, :, 1] = 
2.0  2.0  2.0 
2.0  2.0  2.0 
2.0  2.0  2.0

# 未初期化の配列
# １次元配列
julia> Vector{Int64}(undef, 4) 
4-element Array{Int64,1}: 
4633275264 
4701524592 
4701536352 
4759166960

julia> Matrix{Float32}(undef, 3, 2)
3×2 Array{Float32,2}:
1.13346f-26  1.0f-45
1.0f-45      2.42788f-24
1.13347f-26  1.0f-45

# N次元配列
julia> Array{Float64}(undef, 3, 2)
3×2 Array{Float64,2}:
0.0           1.0e-323
5.0e-324      5.0e-324
2.25608e-314  2.25608e-314

# タプル を使用
julia> Array{Float64}(undef, (2, 3, 1))
2×3×1 Array{Float64,3}:[:, :, 1] =
2.26367e-314  2.32285e-314  2.25639e-314
2.35392e-314  2.32285e-314  2.25639e-314

# 乱数
# 一様分布に従う乱数の生成
# Base.randの使用
julia> rand(3) 
3-element Array{Float64,1}:  
0.8375840717819663  
0.061710442610265925  
0.3356047946945091  

julia> rand(Int32, 3, 2)  
3×2 Array{Int32,2}:  
-1109356720  -1818515624   1611852851   
1979777378   2088060645   -457984219  

julia> rand(Float64, (2, 2))  
2×2 Array{Float64,2}:  
0.32702   0.620583  
0.961646  0.923465

# 要素がBoolの乱数を生成
julia> using Random
julia> bitrand((3, 3))
3×3 BitArray{2}:
0  1  0
1  0  0
1  0  0

# 標準正規分布に従う乱数を生成
# Base.randnを使用
julia> randn(Float64, (3, 3))
3×3 Array{Float64,2}:
0.522662   0.0880295   1.26982
2.0078     1.13166    -0.589239
1.48543   -0.0526412   0.910557

# 縦に結合（Base.cat, Base.vcatの使用、catを使うときはdims=1とする。[ ; ]も使用できる。）
# 配列同士の結合
julia> a = ([0, 1], [2, 3, 4]); vcat(a...) 
5-element Array{Int64,1}: 
0 
1 
2 
3 
4 

julia> b = [0 1 2 4; 3 4 5 6] 
2×4 Array{Int64,2}: 
0  1  2  4 
3  4  5  6

# 行列と行ベクトルの結合
julia> c = [7, 8, 9, 10]'
1×4 Adjoint{Int64,Array{Int64,1}}:
7  8  9  10

julia> vcat(b, c)  
3×4 Array{Int64,2}:
0  1  2   4
3  4  5   6
7  8  9  10

# 行列同士の結合
julia> vcat(b, b) 
4×4 Array{Int64,2}: 
0  1  2  4 
3  4  5  6 
0  1  2  4 
3  4  5  6 

julia> cat(b, b, dims=1) 
4×4 Array{Int64,2}: 
0  1  2  4 
3  4  5  6 
0  1  2  4 
3  4  5  6 

julia> [b; b] 
4×4 Array{Int64,2}: 
0  1  2  4 
3  4  5  6 
0  1  2  4 
3  4  5  6 

# 横に結合（Base.cat, Base.hcatの使用、catを使うときはdims=1とする。[ ; ]も使用できる。）
# 配列同士の結合
julia> a = ([0 1], [2 3 4]); hcat(a...)
1×5 Array{Int64,2}:
0  1  2  3  4

julia> b = ([0, 1, 2, 4], [3, 4, 5, 6]); hcat(b...)  
4×2 Array{Int64,2}:
0  3
1  4
2  5
4  6

# 行列と列ベクトルの結合
julia> c = [0 1; 2 3; 4 5; 4  6] 
4×2 Array{Int64,2}: 
0  1
2  3
4  5
4  6

julia> d = [6, 7, 8, 9] 
4-element Array{Int64,1}: 
6 
7 
8 
9 

julia> hcat(c, d) 
4×3 Array{Int64,2}: 
0  1  6 
2  3  7 
4  5  8 
4  6  9

# 行列同士の連結
julia> hcat(c, c) 
4×4 Array{Int64,2}: 
0  1  0  1 
2  3  2  3 
4  5  4  5 
4  6  4  6

julia> cat(c, c, dims=2)
4×4 Array{Int64,2}:
0  1  0  1
2  3  2  3
4  5  4  5
4  6  4  6

julia> [c c] 
4×4 Array{Int64,2}: 
0  1  0  1 
2  3  2  3 
4  5  4  5 
4  6  4  6

# 次元数を取得する
julia> A = fill(1, (3,4,5,6));
julia> ndims(A)
4

# 形状を取得する
julia> A = fill(1, (3,4,5,6));
julia> size(A)
(3, 4, 5, 6)
julia> size(A, 2)
4

# 全要素数を取得する
julia> length([1, 2, 3, 4, 5])
5

julia> length([1 2 3; 3 4 5])
6

julia> a = ones(3, 3)
3×3 Array{Float64,2}:
1.0  1.0  1.0
1.0  1.0  1.0
1.0  1.0  1.0

# 要素の型
julia> eltype(a) 

# 配列の型
Float64
julia> typeof(a)  
Array{Float64,2}

# 転置
julia> a = Vector(1:4)
4-element Array{Int64,1}:
1
2
3
4

julia> transpose(a)
1×4 LinearAlgebra.Transpose{Int64,Array{Int64,1}}:
1  2  3  4

julia> a'
1×4 LinearAlgebra.Adjoint{Int64,Array{Int64,1}}:
1  2  3  4

julia> b = [1 2; 3 4; 5 6; 7 8]
4×2 Array{Int64,2}:
1  2
3  4
5  6
7  8

julia> transpose(b)
2×4 LinearAlgebra.Transpose{Int64,Array{Int64,2}}:
1  3  5  7
2  4  6  8

julia> b'
2×4 LinearAlgebra.Adjoint{Int64,Array{Int64,2}}:
1  3  5  7
2  4  6  8

# ソート
julia> a = [5, 3, 2, 4, 1]
5-element Array{Int64,1}:
5
3
2
4
1

julia> sort(a)
5-element Array{Int64,1}:
1
2
3
4
5

# 降順のソート
julia> sort(a, rev=true)
5-element Array{Int64,1}:
5 
4
3
2
1

# 行方向のソートは、dims=2を指定する
julia> a = [5 3 2 4 1]
1×5 Array{Int64,2}:
5  3  2  4  1

julia> sort(a, dims=2)
1×5 Array{Int64,2}:
1  2  3  4  5

# 逆順
julia> a = [5, 3, 2, 4, 1]
5-element Array{Int64,1}:
5
3
2
4
1

julia> reverse(a)
5-element Array{Int64,1}:
1
4
2
3
5

julia> a[end:-1:1]
5-element Array{Int64,1}:
1
4
2
3
5

# 条件を満たす要素を抽出
julia> a = reshape(1:16, (4, 4)) 
4×4 reshape(::UnitRange{Int64}, 4, 4) with eltype Int64: 
1  5   9  13 
2  6  10  14 
3  7  11  15 
4  8  12  16 

julia> a[a .< 10] 
9-element Array{Int64,1}: 
1 
2 
3 
4 
5 
6 
7 
8 
9 

julia> a[a .> 10] 
6-element Array{Int64,1}: 
11 
12 
13 
14 
15 
16 

julia> a[a .== 10] 
1-element Array{Int64,1}: 
10

julia> a[a .>= 10]
7-element Array{Int64,1}:
10
11
12
13
14
15
16

julia> a[5 .< a .< 10]
4-element Array{Int64,1}:
6
7
8
9

# 配列の要素を入れ替える
julia> a = [1 3 5 7 9]
1×5 Array{Int64,2}:
1  3  5  7  9

julia> a[2] = 0; a
1×5 Array{Int64,2}:
1  0  5  7  9

julia> a = [1; 3; 5; 7; 9]
5-element Array{Int64,1}:
1
3
5
7
9

julia> a[4] = 0; a
5-element Array{Int64,1}:
1
3
5
0
9

# 条件に基づいて、入れ替える時は、ブロードキャストを使う
julia> a[a .< 4] .= 0; a
5-element Array{Int64,1}:
0
0
5
0
9

# 2次元配列
julia> a = [1 2 3; 4 5 6; 7 8 9]
3×3 Array{Int64,2}:
1  2  3
4  5  6
7  8  9

julia> a = [1 2 3; 4 5 6; 7 8 9]
3×3 Array{Int64,2}:
1  2  3
4  5  6
7  8  9

# ２列目を２に入れ替え
julia> a[:, 2] .= 1; a 
3×3 Array{Int64,2}: 
1  1  3 
4  1  6 
7  1  9

# 複数行の入れ替え：2行目以降を0に入れ替え
julia> a[2:end, :] .= 0; a
3×3 Array{Int64,2}:
1  1  3
0  0  0
0  0  0

julia> a .= 0
3×3 Array{Int64,2}:
0  0  0
0  0  0
0  0  0

# Reshapeで成形したデータは、Array型でないので、そのままでは入れ替えができない。まずArray関数を用いて、Array型に嫌韓してから入れ替えを行う。
julia> a = reshape(1:9, (3, 3)) 
3×3 reshape(::UnitRange{Int64}, 3, 3) with eltype Int64: 
1  4  7 
2  5  8 
3  6  9 

julia> a[2, 2] = 0; a 
ERROR: indexed assignment fails for a reshaped range; consider calling collect Stacktrace: 
julia> a = Array(a) 
3×3 Array{Int64,2}: 
1  4  7 
2  5  8 
3  6  9 

julia> a[2, 2] = 0; a 
3×3 Array{Int64,2}: 
1  4  7 
2  0  8 
3  6  9 

# 最大値、最小値の取得
julia> a = reshape(1:9, (3, 3))
3×3 reshape(::UnitRange{Int64}, 3, 3) with eltype Int64:
1  4  7
2  5  8
3  6  9

# 最大値の取得
julia> maximum(a)  
9

# 最小値の取得
julia> minimum(a)   
1

# 最小値と最大値の両方の取得
julia> extrema(a)   
(1, 9)

# dimsで取得したい最大値(最小値)の列（行）を指定する
julia> a = reshape(1:9, (3, 3)) 
3×3 reshape(::UnitRange{Int64}, 3, 3) with eltype Int64: 
1  4  7 
2  5  8 
3  6  9 

# 列方向の最小値を求める
julia> minimum(a, dims=1) 
1×3 Array{Int64,2}: 
1  4  7 

# 行方向の最小値を求める
julia> minimum(a, dims=2) 
3×1 Array{Int64,2}: 
1 
2 
3

# タプル を用いて、行と列の両方向を指定することも可能
julia> minimum(a, dims=(1, 2)) 
1×1 Array{Int64,2}: 
1 

# 列方向の最大値、最小値を同時に求める
julia> extrema(a, dims=1) 
1×3 Array{Tuple{Int64,Int64},2}: 
(1, 3)  (4, 6)  (7, 9)

julia> a = [1 2 3 4; 3 4 2 1]
2×4 Array{Int64,2}:
1  2  3  4
3  4  2  1

julia> argmax(a)
CartesianIndex(2, 2)

julia> a[CartesianIndex(2, 2)]
4

# dimsで取得したいインデックスの行か列かを指定することができる
julia> a = [1 2; 3 4]
2×2 Array{Int64,2}:
1  2
3  4

julia> argmin(a, dims=1)
1×2 Array{CartesianIndex{2},2}:
CartesianIndex(1, 1)  CartesianIndex(1, 2)

julia> argmax(a, dims=2)
2×1 Array{CartesianIndex{2},2}:
CartesianIndex(1, 2)
CartesianIndex(2, 2)

julia> argmax(a, dims=(1, 2))
1×1 Array{CartesianIndex{2},2}:
CartesianIndex(2, 2)

julia> a[CartesianIndex(2, 2)]
4

# 配列要素のシャフル
julia> using Random
julia> shuffle(1:10)
10-element Array{Int64,1}:
8
6
7
3
2
5
1
9
4
10

julia> a = [1 2 3 4; 3 4 2 1]
2×4 Array{Int64,2}:
1  2  3  4
3  4  2  1

# 列方向にシャフル
julia> a[shuffle(1:end), :]
2×4 Array{Int64,2}:
1  2  3  4
3  4  2  1

# 行方向にシャフル
julia> a[:, shuffle(1:end)]
2×4 Array{Int64,2}:
3	2  3  1
1	4  2  3

# 辞書の定義方法：
julia> fruits = Dict([("Banana" , 1), ("Orange" , 2), ("Apple" , 3)])
Dict{String,Int64} with 3 entries:
"Apple"  => 3"
Orange" => 2"
Banana" => 1

julia> fruits = Dict("Banana" => 1, "Orange" => 2, "Apple" => 3)
Dict{String,Int64} with 3 entries:
"Apple"  => 3
"Orange" => 2
"Banana" => 1

# 要素へアクセスするにはキーを指定する。
julia> fruits["Banana"]
1

# 要素の追加方法
julia> fruits["Peach"] = 4
4
julia> fruits
Dict{String,Int64} with 4 entries:
"Apple"  => 3
"Orange" => 2
"Peach"  => 4
"Banana" => 1

# 集合（Set）の定義方法
julia> fruits = Set(["Banana", "Orange", "Apple"])
Set{String} with 3 elements:
"Apple"
"Orange"
"Banana"

# 集合（Set）の値の追加
julia> push!(fruits, "Mango")
Set{String} with 4 elements:
"Mango"
"Apple"
"Orange"
"Banana"

# 同じ値は追加しても重複しない
julia> push!(fruits, "Apple")
Set{String} with 4 elements:
"Mango"
"Apple"
"Orange"
"Banana"

# 配列への変換
julia> [v for v in fruits] 
4-element Array{String,1}: 
"Mango" 
"Apple" 
"Orange" 
"Banana"

# 部分配列（サブ配列）

julia> A = rand(3,3,2);
julia> A
3×3×2 Array{Float64,3}:
[:, :, 1] =
0.975335   0.840446  0.0250482
0.0178132  0.869445  0.327017
0.65592    0.299449  0.764486
[:, :, 2] =
0.915121  0.108661  0.211831
0.812227  0.420472  0.498775
0.486254  0.13878   0.128309

julia> S1 = view(A, :, 3, 1:2)
3×2 view(::Array{Float64,3}, :, 3, 1:2) with eltype Float64:
0.0250482  0.211831
0.327017   0.498775
0.764486   0.128309

julia> S1 = view(A, 1:2, 1, 1:2)
2×2 view(::Array{Float64,3}, 1:2, 1, 1:2) with eltype Float64:
0.975335   0.915121
0.0178132  0.812227

# 補足：ブロードキャスト

julia> a = [1, 2, 3, 4]
4-element Array{Int64,1}:
1
2
3
4

julia> b = [1 2; 3 4; 5 6; 7 8] 
4×2 Array{Int64,2}: 
1  2 
3  4 
5  6 
7  8

julia> broadcast(+, a, b)
4×2 Array{Int64,2}:
2   3
5   6
8   9
11  12

julia> parse.(Int, ["0", "1"]) 
2-element Array{Int64,1}: 
0 
1
julia> abs.((-4, -2))
(4, 2)

julia> broadcast(+, 1.0, (1.0, -2.0))
(2.0, -1.0)

julia> (+).([[1,3], [0,1]], Ref{Vector{Int}}([2,-3]))
2-element Array{Array{Int64,1},1}:
[3, 0]
[2, -2]

julia> string.(("one","two","three"), ": ", 1:3)
3-element Array{String,1}:
"one: 1"
"two: 2"
"three: 3"

# 補足：リスト内包表記
# 式 for 変数 in イテラブルオブジェクト]


julia> fruits = [i for i in ["apple", "banana", "orange", "mango"]]
4-element Array{String,1}:
"apple"
"banana"
"orange"
"mango" 

julia> [i * j for i in 1:9, j in 1:9]
9×9 Array{Int64,2}:
1   2   3   4   5   6   7   8   9
2   4   6   8  10  12  14  16  18
3   6   9  12  15  18  21  24  27
4   8  12  16  20  24  28  32  36
5  10  15  20  25  30  35  40  45
6  12  18  24  30  36  42  48  54
7  14  21  28  35  42  49  56  63
8  16  24  32  40  48  56  64  72
9  18  27  36  45  54  63  72  81

# 補足：パイプライン処理

julia> [1, 4, 1, 3, 2, 5] |> sort |> unique |> length
5

# モジュール
######################
using Lib

using BigLib: thing1, thing2

import Base.show

julia> using Statistics
julia> mean(1:5)
3.0

julia> mean([1:5 6:10])
5.5

julia> mean([1, missing, 3, missing, 5])
missing

julia> mean(skipmissing([1, missing, 3, missing, 5]))
3.0

julia> using Statistics: mean, var, std 

julia> mean(1:10) 
5.5 

julia> var(1:10) 
9.166666666666666 

julia> std(1:10) 
3.0276503540974917

julia> module MyModule
       export x, y

       x() = "a"
       y() = "b"
       p() = "c"

       end

julia> using Main.MyModule

julia> MyModule.x()
"a"

# 複数ファイルでの記述

module Foo1
include("file1.jl")
include("file2.jl")
include("file3.jl")
end

module Normal1
include("mycode1.jl")
include("mycode2.jl")
end
module Testing1
include("safe_operators.jl")
include("mycode1.jl")
end

julia> module Level1
       export run1,Level2
       function run1()
           println("toprunner")
       end
           module Level2
               using ..Level1
           export run2,Level3
           function run2()
               println("toprunner2")
               run1()
           end
               module Level3
                   using ...Level1
                   using ..Level2
                   export run3
                   function run3()
                       println("toprunner3")
                       run2()
                       run1()
                       println("winner!")
                   end
               end
           end
       end

julia> using .Level1
julia> using .Level2
julia> using .Level3
julia> run1()
toprunner

julia> run2()
toprunner2
toprunner

# メタプログラミング
julia> f = "10 + x"
"10 + x"

julia> ex1 = Meta.parse(f)
:(10 + x)

julia> typeof(ex1)
Expr

julia> dump(ex1)
Expr
 head: Symbol call
 args: Array{Any}((3,))
  1: Symbol +
  2: Int64 10
  3: Symbol x

julia> ex1.head
:call

julia> ex1.args
3-element Array{Any,1}:
   :+
  10
   :x

julia> Symbol("data",1)
:data1

julia> Symbol(:all,'_',"data")
:all_data

以下のように、Expr()関数を用いることでExpr型を直接表現することも可能です。

julia> ex2 = Expr(:call,:+,10,:x)
:(10 + x)

julia> dump(ex2)
Expr
  head: Symbol call
  args: Array{Any}((3,))
    1: Symbol +
    2: Int64 10
    3: Symbol x

julia> ex1 == ex2
true

julia> dump(:(10 + 20))
Expr
  head: Symbol call
  args: Array{Any}((3,))
    1: Symbol +
    2: Int64 10
    3: Int64 20

julia> dump(:(x = 20))
Expr
  head: Symbol =
  args: Array{Any}((2,))
    1: Symbol x
    2: Int64 20

julia> ex5 = quote
           10 + 20
       end
quote
    #= REPL[6]:2 =#
    10 + 20
end

julia> dump(:(3x - 20))
Expr
  head: Symbol call
  args: Array{Any}((3,))
    1: Symbol –
    2: Expr
      head: Symbol call
      args: Array{Any}((3,))
        1: Symbol *
        2: Int64 3
        3: Symbol x
    3: Int64 20

julia> x = 10

;julia> ex = :($x * y)
:(10y)

# タプルなどで補間することもできます。
julia> ex = :(a in $:((4,5,6)) )
:(a in (4, 5, 6))

# 補間するものがシンボルの場合は、前に : をつけ、:x, :yのようにする。
julia> args = [:x, :y];
julia> :(f($(args...), 100))
:(f(x, y, 100))

# さらに、以下のように関数Meta.parse（）で処理した結果、すなわちExpr型の抽象構文木は、関数eval()で評価できます。
# また、以下のようにExpr型は入れ子構造をとることもできます。

julia> ex3 = Meta.parse("(5500 + 550) / 55") 
:((5500 + 550) / 55) 
julia> dump(ex3) 
Expr 
 head: Symbol call 
 args: Array{Any}((3,)) 
  1: Symbol / 
  2: Expr 
   head: Symbol call 
   args: Array{Any}((3,)) 
    1: Symbol + 
    2: Int64 5500 
    3: Int64 550 
  3: Int64 55 

julia> eval(ex3)
110.0

# 変数を含む式をeval()関数で評価する場合は、変数に予め値を代入しておく必要がある。
julia> ex = :(x * y)
:( x * y)

julia> eval(ex)
ERROR: UndefVarError: b not defined[...]

julia> x = 2; y = 5;

julia> eval(ex)
10

# マクロの作成
# マクロの作成は以下のようにします。

julia> macro helloworld()
           return :( println("Hello, world!") )
       end
@helloworld (macro with 1 method)

# 作成したマクロは以下のようにして実行します。

julia> @helloworld()
Hello, world!

# 引数をとるマクロを作成することもできます。
julia> macro add(x,y)
          return :($x + $y)
       end
@add (macro with 1 method)

julia> @add(10,20)
30

# Macroexpand関数を使って、実行したマクロのExpr型抽象構文木を表示できます。
julia> ex = macroexpand(Main, :(@add(10,20))) 
:(10 + 20) 

julia> typeof(ex) 
Expr

# 外部プログラムの呼び出し
###################

# C言語の呼び出し

julia> t = ccall(:clock, Int64, ())
1030299

julia> t
1030299

julia> typeof(t)
Int64

julia> t = ccall(:sqrt, Cdouble, (Cdouble,), 10.0)
3.1622776601683795

# ポインタの受け渡し
julia> path = ccall(:getenv, Cstring, (Cstring,), "SHELL")
Cstring(0x00007ffee3af8d52)

julia> unsafe_string(path)
"/bin/zsh"

julia> path = ccall(:getenv, Cstring, (Cstring,), "HOME")
Cstring(0x00007ffee3af8d60)

julia> unsafe_string(path)
"/Users/kazuoishii"

# シェルのコマンドの呼び出し

julia> `pwd`
`pwd`

julia> run(`pwd`)
/Users/kazuoishii
Process(`pwd`, ProcessExited(0))

julia> pipeline(`pwd`,`wc`)
pipeline(`pwd`, stdout=`wc`)

julia> run(ans)
1       1      18
Base.ProcessChain(Base.Process[Process(`pwd`, ProcessExited(0)), Process(`wc`, ProcessExited(0))], Base.DevNull(), Base.DevNull(), Base.DevNull())

# パッケージ
# REPLのパッケージモードを使う場合
# パッケージのインストール

(@v1.5) pkg> add パッケージ名

(@v1.5) pkg> add パッケージ名１ パッケージ名2 パッケージ名3

# パッケージの削除

(@v1.5) pkg> rm パッケージ名

# インストール済パッケージの確認

(@v1.5) pkg> status

# パッケージのアップデート

(v1.5) pkg> update

(v1.5) pkg> up




