################################
# 3章　統計解析
################################
# 線形代数（LinearAlgebra）
# 対角和（関数tr）、行列式（関数det）、逆行列（関数inv）

julia> using LinearAlgebra
julia> A = [1 2 3 4; 5 1 5 8; 9 10 7 1; 13 8 1 16]
4×4 Array{Int64,2}:
1   2  3   4
5   1  5   8
9  10  7  11
13   8  1  16

julia> tr(A)
25

julia> det(A)
2742.0

julia> inv(A)
4×4 Array{Float64,2}:
-0.370897    0.152443   0.04814     0.0134938
 0.275711   -0.222465   0.0350109   0.0401167
 0.0601751   0.124726   0.0393873  -0.0798687
 0.159737   -0.020423  -0.059081    0.0364697

# 固有値、固有ベクトルは以下のようにして求めることができます。

julia> A = [3 2; 4 1] 
2×2 Array{Int64,2}: 
3  2 
4  1 

julia> eigvals(A) 
2-element Array{Float64,1}: 
-1.0 
5.0 

julia> eigvecs(A) 
2×2 Array{Float64,2}: 
-0.447214  0.707107 
 0.894427  0.707107

# 特殊行列：
# 対称行列

Julia> A = [1 0 3 0 2; 0 5 0 4 0; 7 0 6 0 8; 0 9 0 1 0; 2 0 7 0 5] 
5×5 Array{Int64,2}: 
1  0  3  0  2 
0  5  0  4  0 
7  0  6  0  8 
0  9  0  1  0 
2  0  7  0  5 

julia> Supper = Symmetric(A) 
5×5 Symmetric{Int64,Array{Int64,2}}: 
1  0  3  0  2 
0  5  0  4  0 
3  0  6  0  8 
0  4  0  1  0 
2  0  8  0  5 

julia> Lower = Symmetric(A, :L) 
5×5 Symmetric{Int64,Array{Int64,2}}: 
1  0  7  0  2 
0  5  0  9  0 
7  0  6  0  7 
0  9  0  1  0 
2  0  7  0  5

julia> issymmetric(Symmetric(A, :L))
true

# エルミート行列

julia> A = [1-1im 3+3im 2+2im 4-4im 3-3im; 1+1im 4-4im 3-3im 5+5im 1+1im; 6-6im 1+1im 7-7im 1-1im 8+8im; 2-2im 9+9im 3+3im 1+1im 2-2im; 2+2im 7+7im 3-3im 6-6im 4+4im]

;julia> A
5×5 Array{Complex{Int64},2}:
1-1im  3+3im  2+2im  4-4im  3-3im
1+1im  4-4im  3-3im  5+5im  1+1im
6-6im  1+1im  7-7im  1-1im  8+8im
2-2im  9+9im  3+3im  1+1im  2-2im
2+2im  7+7im  3-3im  6-6im  4+4im

julia> Hupper = Hermitian(A)
5×5 Hermitian{Complex{Int64},Array{Complex{Int64},2}}:
1+0im  3+3im  2+2im  4-4im  3-3im
3-3im  4+0im  3-3im  5+5im  1+1im
2-2im  3+3im  7+0im  1-1im  8+8im
4+4im  5-5im  1+1im  1+0im  2-2im
3+3im  1-1im  8-8im  2+2im  4+0im

julia> Hlower = Hermitian(A, :L)
5×5 Hermitian{Complex{Int64},Array{Complex{Int64},2}}:
1+0im  1-1im  6+6im  2+2im  2-2im
1+1im  4+0im  1-1im  9-9im  7-7im
6-6im  1+1im  7+0im  3-3im  3+3im
2-2im  9+9im  3+3im  1+0im  6+6im
2+2im  7+7im  3-3im  6-6im  4+0im

julia> ishermitian(Hermitian(A, :L))
true

# 単位対角の下三角行列（UnitLowerTriangular）、単位対角の上三角行列（UnitUpperTriangular）、上ヘッセンベルク行列（UpperHessenberg）

julia> A = [3.0 2.0 1.0; 4.0 5.0 7.0; 6.0 9.0 8.0]
3×3 Array{Float64,2}:
3.0  2.0  1.0
4.0  5.0  7.0
6.0  9.0  8.0

julia> UnitLowerTriangular(A)
3×3 UnitLowerTriangular{Float64,Array{Float64,2}}:
1.0   ⋅    ⋅
4.0  1.0   ⋅
6.0  9.0  1.0

julia> UnitUpperTriangular(A)
3×3 UnitUpperTriangular{Float64,Array{Float64,2}}:
1.0  2.0  1.0
⋅    1.0  7.0
⋅    ⋅    1.0

julia> UpperHessenberg(A)
3×3 UpperHessenberg{Float64,Array{Float64,2}}:
3.0  2.0  1.0
4.0  5.0  7.0
⋅    9.0  8.0

# 行列の演算

julia> a = [1 2 3 4; 4 5 6 7; 7 8 9 10; 11 12 13 14]
4×4 Array{Int64,2}:
1   2   3    4
4   5   6    7
7   8   9   10
11  12  13  14

julia> b = [1 2 3 4; -4 5 -6 7; 7 -8 9 -10; -11 12 -13 14]
4×4 Array{Int64,2}:
  1   2    3    4
 -4   5   -6    7
  7  -8    9  -10
-11  12  -13   14

# 行列の足し算は + で実施します。
julia> a + b #行列の足し算
4×4 Array{Int64,2}:
 2   4   6   8
 0  10   0  14
14   0  18   0
 0  24   0  28

# 行列の引き算は – で実施します。
julia> a – b #行列の引き算
4×4 Array{Int64,2}:
0   0   0   0
8   0  12   0
0  16   0  20
22   0  26   0

# 行列の積は * で実施します。
julia> a * b #行列の積
4×4 Array{Int64,2}:
-30   36   -34   44
-51   69   -55   89
-72  102   -76  134
-100  146  -104  194

#  .*  は行列の要素毎の掛け算を意味します。
julia> a .* b # 行列の要素毎の掛け算
4×4 Array{Int64,2}:
1    4     9    16
-16   25   -36    49
49  -64    81  -100
-121  144  -169   196

# a ./ b は行列の要素毎の割り算を意味します。
julia> a ./ b # 行列の要素毎の割り算
4×4 Array{Float64,2}:
 1.0   1.0   1.0   1.0
-1.0   1.0  -1.0   1.0
 1.0  -1.0   1.0  -1.0
-1.0   1.0  -1.0   1.0


a / cは、a * inv(c)と等価です。

julia> c = a + b
4×4 Array{Int64,2}:
  2   4   6   8
  0  10   0  14
 14   0  18   0
  0  24   0  28

julia> a / c
4×4 Array{Float64,2}:
  0.5       -0.0       0.0       0.0
  0.25       0.142857  0.25      0.107143
  0.0        0.285714  0.5       0.214286
 -0.333333   0.47619   0.833333  0.357143

julia> a * inv(c)
4×4 Array{Float64,2}:
  0.5          -2.22045e-16  0.0       0.0
  0.25          0.142857     0.25      0.107143
  4.44089e-16   0.285714     0.5       0.214286
 -0.333333      0.47619      0.833333  0.357143

＊だたし、上記の１行２列の値は、浮動小数点数型による誤差が生じている。


# a \ bは、inv(a) * bと等価です。
julia> a \ b
4×4 Array{Float64,2}:
9.0072e15   -9.0072e15   1.35108e16  -1.1259e16
-1.05835e16   9.0072e15  -1.82396e16   1.1259e16
-5.85468e15   9.0072e15  -4.05324e15   1.1259e16
7.43094e15  -9.0072e15   8.78202e15  -1.1259e16

julia> inv(a) * b
4×4 Array{Float64,2}:
9.0072e15   -9.00720e15   1.35108e16  -1.1259e16
-1.05835e16   9.0072e15   -1.82396e16   1.1259e16
-5.85468e15   9.0072e15   -4.05324e15   1.1259e16
7.43094e15  -9.0072e15    8.78202e15  -1.1259e16

# 転置行列
# 転置行列は ' を利用してx'のように記述します。または, transpose()を使用します。

julia> A = [1 3 2; 6 5 4; 7 9 8] 
3×3 Array{Int64,2}: 
1  3  2 
6  5  4 
7  9  8

julia> A'
3×3 Adjoint{Int64,Array{Int64,2}}:
1  6  7
3  5  9
2  4  8

julia> transpose(A)
3×3 Transpose{Int64,Array{Int64,2}}:
1  6  7
3  5  9
2  4  8

# 行列分解
# LU分解の実施例

julia> A = rand(Float64, (4, 4))*10
4×4 Array{Float64,2}:
6.34614  9.66868  0.596454  5.81881
3.92662  3.51592  6.34389   2.99955
6.45179  7.11345  0.880595  3.68593
5.98174  1.55886  0.480329  8.51611

julia> factorize(A)
LU{Float64,Array{Float64,2}}
L factor:4×4 Array{Float64,2}:
1.0        0.0        0.0        0.0
0.927144   1.0        0.0        0.0
0.608609   0.161503   1.0        0.0
0.983624  -0.53049   -0.0764252  1.0
U factor:4×4 Array{Float64,2}:
6.45179   7.11345   0.880595   3.68593
0.0      -5.03633  -0.336109   5.09873
0.0       0.0       5.86224   -0.067197
0.0       0.0       0.0        4.89293

# ベクトルの内積、外積とノルム

julia> A = 10*rand(Float64, (1, 4))
1×4 Array{Float64,2}:
0.317578  5.55968  2.60586  6.95207

julia> B = 10*rand(Float64, (1, 4))
1×4 Array{Float64,2}:
3.88593  2.56117  2.05317  0.145272

# ベクトルの内積
julia> dot(A,B)
21.83359133466677

# ベクトルの内積
julia> A ⋅ B 
21.83359133466677

# ベクトルの外積
julia> cross([1,2,3],[4,5,6])
3-element Array{Int64,1}:
-3
6
-3

# ベクトルの外積
julia> ×([1,2,3],[4,5,6])
3-element Array{Int64,1}:
-3
6
-3

# L1ノルム
julia> norm(A, 1)
15.435196787136928

# L2ノルム
julia> norm(A, 2)
9.280773726389617

# L1ノルムによる正規化
julia> normalize(A, 1)
1×4 Array{Float64,2}:
0.0205749  0.360195  0.168826  0.450404

# L2ノルムによる正規化
julia> normalize(A, 2)
1×4 Array{Float64,2}:
0.0342189  0.599053  0.280781  0.749083

# データフレーム（DataFrames）

(@v1.5) pkg> add DataFrames

julia> using DataFrames

# データフレームの作成 I
julia> df = DataFrame(A = 1:10, B = ["A", "A", "B", "B", "B", "C", "A", "B", "B", "C"], C = ["M", "M", "F","F","F","M","F","M","F", "M"])
10×3 DataFrame
│ Row │ A     │ B      │ C      │
│     │ Int64 │ String │ String │
├─────┼───────┼────────┼────────┤
│ 1   │ 1     │ A      │ M      │
│ 2   │ 2     │ A      │ M      │
│ 3   │ 3     │ B      │ F      │
│ 4   │ 4     │ B      │ F      │
│ 5   │ 5     │ B      │ F      │
│ 6   │ 6     │ C      │ M      │
│ 7   │ 7     │ A      │ F      │
│ 8   │ 8     │ B      │ M      │
│ 9   │ 9     │ B      │ F      │
│ 10  │ 10    │ C      │ M      │

# 特定の列の抽出
# A列の抽出
julia> df.A
10-element Array{Int64,1}:
1
2
3
4
5
6
7
8
9
10

julia> df."A"
10-element Array{Int64,1}:
1
2
3
4
5
6
7
8
9
10

# 列名の抽出
julia> names(df)
3-element Array{String,1}:
"A"
"B"
"C"

# シンボル型による列名の抽出
julia> propertynames(df)
3-element Array{Symbol,1}:
:A
:B
:C

# データフレームの作成(列ベース) II
# データフレームの枠の作成
julia> df2 = DataFrame()
0×0 DataFrame

# A列の作成
julia> df2.A = 1:10
1:10

# B列の作成
julia> df2.B = ["A","A","B","B","B","C","A","B","B","C"]
10-element Array{String,1}:
"A"
"A"
"B"
"B"
"B"
"C"
"A"
"B"
"B"
"C"

# C列の作成
julia> df2.C = ["M","M","F","F","F","M","F","M","F","M"]
10-element Array{String,1}:
"M"
"M"
"F"
"F"
"F"
"M"
"F"
"M"
"F"
"M"

# データフレームdfの表示
julia> df2
10×3 DataFrame
│ Row │ A     │ B      │ C      │
│     │ Int64 │ String │ String │
├─────┼───────┼────────┼────────┤
│ 1   │ 1     │ A      │ M      │
│ 2   │ 2     │ A      │ M      │
│ 3   │ 3     │ B      │ F      │
│ 4   │ 4     │ B      │ F      │
│ 5   │ 5     │ B      │ F      │
│ 6   │ 6     │ C      │ M      │
│ 7   │ 7     │ A      │ F      │
│ 8   │ 8     │ B      │ M      │
│ 9   │ 9     │ B      │ F      │
│ 10  │ 10    │ C      │ M      │

# 行サイズの表示
julia> size(df2, 1)
10

# 列サイズの表示
julia> size(df2, 2)
3

# 行サイズ、列サイズの表示
julia> size(df2)
(10, 3)

# データフレームの作成(行ベース) III
julia> df3 = DataFrame(A = Int[], B = String[], C = String[])
0×3 DataFrame

# 1列目の追加
julia> push!(df3, (1, "A", "M"))
1×3 DataFrame
│ Row │ A     │ B      │ C      │
│     │ Int64 │ String │ String │
├─────┼───────┼────────┼────────┤
│ 1   │ 1     │ A      │ M      │

# 2列目の追加
julia> push!(df3, (2, "B", "M"))
2×3 DataFrame
│ Row │ A     │ B      │ C      │
│     │ Int64 │ String │ String │
├─────┼───────┼────────┼────────┤
│ 1   │ 1     │ A      │ M      │
│ 2   │ 2     │ B      │ M      │

# 3列目の追加
julia> push!(df3, (3, "C", "F"))
3×3 DataFrame
│ Row │ A     │ B      │ C      │
│     │ Int64 │ String │ String │
├─────┼───────┼────────┼────────┤
│ 1   │ 1     │ A      │ M      │
│ 2   │ 2     │ B      │ M      │
│ 3   │ 3     │ C      │ F      │

# データフレームの作成IV（行列からデータフレームに変換）
# １）行列の作成
julia> mat = [x+y for x in 2:8, y in 3:5]
7×3 Array{Int64,2}:
5   6   7
6   7   8
7   8   9
8   9  10
9  10  11
10  11  12
11  12  13

# 2) 行列からデータフレームへの変換
julia> df4 = DataFrame(mat)
7×3 DataFrame
│ Row │ x1    │ x2    │ x3    │
│     │ Int64 │ Int64 │ Int64 │
├─────┼───────┼───────┼───────┤
│ 1   │ 5     │ 6     │ 7     │
│ 2   │ 6     │ 7     │ 8     │
│ 3   │ 7     │ 8     │ 9     │
│ 4   │ 8     │ 9     │ 10    │
│ 5   │ 9     │ 10    │ 11    │
│ 6   │ 10    │ 11    │ 12    │
│ 7   │ 11    │ 12    │ 13    │

# 3) Symbol型によるラベルの付与
julia> df4 = DataFrame(mat, [:A, :B, :C])
7×3 DataFrame
│ Row │ A     │ B     │ C     │
│     │ Int64 │ Int64 │ Int64 │
├─────┼───────┼───────┼───────┤
│ 1   │ 5     │ 6     │ 7     │
│ 2   │ 6     │ 7     │ 8     │
│ 3   │ 7     │ 8     │ 9     │
│ 4   │ 8     │ 9     │ 10    │
│ 5   │ 9     │ 10    │ 11    │
│ 6   │ 10    │ 11    │ 12    │
│ 7   │ 11    │ 12    │ 13    │

# 上位3行の表示
julia> first(df, 3)
3×3 DataFrame
│ Row │ A     │ B      │ C      │
│     │ Int64 │ String │ String │
├─────┼───────┼────────┼────────┤
│ 1   │ 1     │ A      │ M      │
│ 2   │ 2     │ A      │ M      │
│ 3   │ 3     │ B      │ F      │

# 位3行の表示
julia> last(df, 3)3×3 DataFrame
│ Row │ A     │ B      │ C      │
│     │ Int64 │ String │ String │
├─────┼───────┼────────┼────────┤
│ 1   │ 8     │ B      │ M      │
│ 2   │ 9     │ B      │ F      │
│ 3   │ 10    │ C      │ M      │

# 部分集合の抽出
# 2行目から4行目までの抽出
julia> df[2:4, :]
3×3 DataFrame
│ Row │ A     │ B      │ C      │
│     │ Int64 │ String │ String │
├─────┼───────┼────────┼────────┤
│ 1   │ 2     │ A      │ M      │
│ 2   │ 3     │ B      │ F      │
│ 3   │ 4     │ B      │ F      │

# 2行目から4行目まで、およびA列、B列の抽出
julia> df[2:4, [:A, :B]]
3×2 DataFrame
│ Row │ A     │ B      │
│     │ Int64 │ String │
├─────┼───────┼────────┤
│ 1   │ 2     │ A      │
│ 2   │ 3     │ B      │
│ 3   │ 4     │ B      │

# データフレームの作成V（乱数を用いて行列を作成し、データフレームに変換）
julia> using DataFrames, Random
julia> Random.seed!(1);
julia> df = DataFrame(rand(10, 4)*10);

julia> df
10×4 DataFrame
│ Row │ x1        │ x2       │ x3       │ x4       │
│     │ Float64   │ Float64  │ Float64  │ Float64  │
├─────┼───────────┼──────────┼──────────┼──────────┤
│ 1   │ 2.36033   │ 5.55751  │ 0.769509 │ 4.6335   │
│ 2   │ 3.46517   │ 4.37108  │ 6.40396  │ 1.85821  │
│ 3   │ 3.12707   │ 4.24718  │ 8.73544  │ 1.11981  │
│ 4   │ 0.0790928 │ 7.73223  │ 2.78582  │ 9.76312  │
│ 5   │ 4.88613   │ 2.8119   │ 7.51313  │ 0.516146 │
│ 6   │ 2.10968   │ 2.09472  │ 6.44883  │ 5.3803   │
│ 7   │ 9.51916   │ 2.51379  │ 0.778264 │ 4.55692  │
│ 8   │ 9.99905   │ 0.203749 │ 8.48185  │ 2.79395  │
│ 9   │ 2.51662   │ 2.87702  │ 0.856352 │ 1.78246  │
│ 10  │ 9.86666   │ 8.59512  │ 5.53206  │ 5.48983  │

# データフレームの行数の表示
julia> nrow(df)
10

# データフレームの列数の表示
julia> ncol(df)
4

# データフレームの行数および列数の表示
julia> size(df)
(10, 4)

# タプルを用いた、行数、列数、およびその両方の表示
julia> nrow(df), ncol(df), size(df)
(10, 4, (10, 4))

# 関数describe（）により、変数（x1, x2, x3, x4）ごとの要約統計量（平均、最小値、中央値、最大値など）を表示。
julia> describe(df) 
4×8 DataFrame 
│ Row │ variable │ mean    │ min       │ median  │ max     │ nunique │ nmissing │ eltype   │
│     │ Symbol   │ Float64 │ Float64   │ Float64 │ Float64 │ Nothing │ Nothing  │ DataType │ ├─────┼──────────┼─────────┼───────────┼─────────┼─────────┼─────────┼──────────┼──────────┤ 
│ 1   │ x1       │ 4.7929  │ 0.0790928 │ 3.29612 │ 9.99905 │         │          │ Float64  │ 
│ 2   │ x2       │ 4.10043 │ 0.203749  │ 3.5621  │ 8.59512 │         │          │ Float64  │ 
│ 3   │ x3       │ 4.83052 │ 0.769509  │ 5.96801 │ 8.73544 │         │          │ Float64  │ 
│ 4   │ x4       │ 3.78942 │ 0.516146  │ 3.67544 │ 9.76312 │         │          │ Float64  │

describe(df)のうち、変数にx2, x4を含む文字列のみを表示する。
julia> filter(x -> occursin(r"[24]$", String(x[:variable])),describe(df))
2×8 DataFrame
│ Row │ variable │ mean    │ min      │ median  │ max     │ nunique │ nmissing │ eltype   │
│     │ Symbol   │ Float64 │ Float64  │ Float64 │ Float64 │ Nothing │ Nothing  │ DataType │
├─────┼──────────┼─────────┼──────────┼─────────┼─────────┼─────────┼──────────┼──────────┤
│ 1   │ x2       │ 4.10043 │ 0.203749 │ 3.5621  │ 8.59512 │         │          │ Float64  │
│ 2   │ x4       │ 3.78942 │ 0.516146 │ 3.67544 │ 9.76312 │         │          │ Float64  │

# データベース様式の結合
# データフレームの結合には、内部結合（innerjoin）、完全外部結合（outerjoin）、左外部結合（leftjoin）、右外部結合（rightjoin）、準結合（semijoin）、アンチ結合（antijoin）、交差結合（crossjoin）を実施できる。以下のそれぞれの実施例を示す。

julia> people = DataFrame(ID = [1, 2], Name = ["Taro", "Jiro"])
2×2 DataFrame
│ Row │ ID    │ Name   │
│     │ Int64 │ String │
├─────┼───────┼────────┤
│ 1   │ 1     │ Taro   │
│ 2   │ 2     │ Jiro   │

julia> jobs = DataFrame(ID = [1, 3], Job = ["Doctor", "Nurse"])
2×2 DataFrame
│ Row │ ID    │ Job    │
│     │ Int64 │ String │
├─────┼───────┼────────┤
│ 1   │ 1     │ Doctor │
│ 2   │ 3     │ Nurse  │

# 内部結合（innerjoin）
julia> innerjoin(people, jobs, on = :ID)
1×3 DataFrame
│ Row │ ID    │ Name   │ Job    │
│     │ Int64 │ String │ String │
├─────┼───────┼────────┼────────┤
│ 1   │ 1     │ Taro   │ Doctor │

# 完全外部結合（outerjoin）
julia> outerjoin(people, jobs, on = :ID)
3×3 DataFrame
│ Row │ ID    │ Name    │ Job     │
│     │ Int64 │ String? │ String? │
├─────┼───────┼─────────┼─────────┤
│ 1   │ 1     │ Taro    │ Doctor  │
│ 2   │ 2     │ Jiro    │ missing │
│ 3   │ 3     │ missing │ Nurse   │

# 左外部結合（leftjoin）
julia> leftjoin(people, jobs, on = :ID)
2×3 DataFrame
│ Row │ ID    │ Name   │ Job     │
│     │ Int64 │ String │ String? │
├─────┼───────┼────────┼─────────┤
│ 1   │ 1     │ Taro   │ Doctor  │
│ 2   │ 2     │ Jiro   │ missing │

# 右外部結合（rightjoin）
julia> rightjoin(people, jobs, on = :ID)
2×3 DataFrame
│ Row │ ID    │ Name    │ Job    │
│     │ Int64 │ String? │ String │
├─────┼───────┼─────────┼────────┤
│ 1   │ 1     │ Taro    │ Doctor │
│ 2   │ 3     │ missing │ Nurse  │

# 準結合（semijoin）
julia> semijoin(people, jobs, on = :ID)
1×2 DataFrame
│ Row │ ID    │ Name   │
│     │ Int64 │ String │
├─────┼───────┼────────┤
│ 1   │ 1     │ Taro   │

# アンチ結合（antijoin）
julia> antijoin(people, jobs, on = :ID)
1×2 DataFrame
│ Row │ ID    │ Name   │
│     │ Int64 │ String │
├─────┼───────┼────────┤
│ 1   │ 2     │ Jiro   │

# 交差結合（crossjoin）
julia> crossjoin(people, jobs, makeunique = true)
4×4 DataFrame
│ Row │ ID    │ Name   │ ID_1  │ Job    │
│     │ Int64 │ String │ Int64 │ String │
├─────┼───────┼────────┼───────┼────────┤
│ 1   │ 1     │ Taro   │ 1     │ Doctor │
│ 2   │ 1     │ Taro   │ 3     │ Nurse  │
│ 3   │ 2     │ Jiro   │ 1     │ Doctor │
│ 4   │ 2     │ Jiro   │ 3     │ Nurse  │

# データのソート（並べ替え）は、関数sort!を用いて以下のようにして行う。
# データフレームの作成
julia> df = DataFrame(rand(10, 4)*10)
10×4 DataFrame
│ Row │ x1       │ x2        │ x3       │ x4       │
│     │ Float64  │ Float64   │ Float64  │ Float64  │
├─────┼──────────┼───────────┼──────────┼──────────┤
│ 1   │ 8.93776  │ 0.219847  │ 1.63631  │ 1.37288  │
│ 2   │ 3.45127  │ 7.70197   │ 9.27262  │ 8.35346  │
│ 3   │ 4.91693  │ 4.16282   │ 0.702445 │ 0.989324 │
│ 4   │ 3.60103  │ 6.16441   │ 7.21108  │ 1.06268  │
│ 5   │ 5.49898  │ 6.54443   │ 8.37729  │ 1.72115  │
│ 6   │ 4.12312  │ 5.02658   │ 2.80922  │ 5.27355  │
│ 7   │ 8.79457  │ 0.0678417 │ 4.11445  │ 0.185918 │
│ 8   │ 0.838767 │ 3.72687   │ 0.537688 │ 7.86598  │
│ 9   │ 2.45625  │ 5.87098   │ 2.12775  │ 5.04908  │
│ 10  │ 6.08269  │ 1.55853   │ 7.28529  │ 2.98947  │

# 昇順による並べ替え
julia> sort!(df)10×4 DataFrame
│ Row │ x1       │ x2        │ x3       │ x4       │
│     │ Float64  │ Float64   │ Float64  │ Float64  │
├─────┼──────────┼───────────┼──────────┼──────────┤
│ 1   │ 0.838767 │ 3.72687   │ 0.537688 │ 7.86598  │
│ 2   │ 2.45625  │ 5.87098   │ 2.12775  │ 5.04908  │
│ 3   │ 3.45127  │ 7.70197   │ 9.27262  │ 8.35346  │
│ 4   │ 3.60103  │ 6.16441   │ 7.21108  │ 1.06268  │
│ 5   │ 4.12312  │ 5.02658   │ 2.80922  │ 5.27355  │
│ 6   │ 4.91693  │ 4.16282   │ 0.702445 │ 0.989324 │
│ 7   │ 5.49898  │ 6.54443   │ 8.37729  │ 1.72115  │
│ 8   │ 6.08269  │ 1.55853   │ 7.28529  │ 2.98947  │
│ 9   │ 8.79457  │ 0.0678417 │ 4.11445  │ 0.185918 │
│ 10  │ 8.93776  │ 0.219847  │ 1.63631  │ 1.37288  │

# 降順による並べ替え
julia> sort!(df, rev = true)
10×4 DataFrame
│ Row │ x1       │ x2        │ x3       │ x4       │
│     │ Float64  │ Float64   │ Float64  │ Float64  │
├─────┼──────────┼───────────┼──────────┼──────────┤
│ 1   │ 8.93776  │ 0.219847  │ 1.63631  │ 1.37288  │
│ 2   │ 8.79457  │ 0.0678417 │ 4.11445  │ 0.185918 │
│ 3   │ 6.08269  │ 1.55853   │ 7.28529  │ 2.98947  │
│ 4   │ 5.49898  │ 6.54443   │ 8.37729  │ 1.72115  │
│ 5   │ 4.91693  │ 4.16282   │ 0.702445 │ 0.989324 │
│ 6   │ 4.12312  │ 5.02658   │ 2.80922  │ 5.27355  │
│ 7   │ 3.60103  │ 6.16441   │ 7.21108  │ 1.06268  │
│ 8   │ 3.45127  │ 7.70197   │ 9.27262  │ 8.35346  │
│ 9   │ 2.45625  │ 5.87098   │ 2.12775  │ 5.04908  │
│ 10  │ 0.838767 │ 3.72687   │ 0.537688 │ 7.86598  │

# 列を指定して並べ替え
# ここでは、x2を指定して並べ替えを行なっている。
julia> sort!(df, [:x2])
0×4 DataFrame
│ Row │ x1       │ x2        │ x3       │ x4       │
│     │ Float64  │ Float64   │ Float64  │ Float64  │
├─────┼──────────┼───────────┼──────────┼──────────┤
│ 1   │ 8.79457  │ 0.0678417 │ 4.11445  │ 0.185918 │
│ 2   │ 8.93776  │ 0.219847  │ 1.63631  │ 1.37288  │
│ 3   │ 6.08269  │ 1.55853   │ 7.28529  │ 2.98947  │
│ 4   │ 0.838767 │ 3.72687   │ 0.537688 │ 7.86598  │
│ 5   │ 4.91693  │ 4.16282   │ 0.702445 │ 0.989324 │
│ 6   │ 4.12312  │ 5.02658   │ 2.80922  │ 5.27355  │
│ 7   │ 2.45625  │ 5.87098   │ 2.12775  │ 5.04908  │
│ 8   │ 3.60103  │ 6.16441   │ 7.21108  │ 1.06268  │
│ 9   │ 5.49898  │ 6.54443   │ 8.37729  │ 1.72115  │
│ 10  │ 3.45127  │ 7.70197   │ 9.27262  │ 8.35346  │

# 欠損値の処理
# 欠損値を含むレコードは、以下のように関数dropmissing（）を用いて除くことができる。

# 欠損値を含むデータフレームの作成
julia> df = DataFrame(ID = 1:7,
name = ["Taro", "Jiro", "Saburo", "Hanako", "Kyoko", "Tomoko", "Akemi"],                                          
sex = ["M", missing, "M", "F", "F", missing, "F"],      
height = [170, 165, 175, 150, missing, 160, missing],    
weight = [80, missing, missing, 40, 45, missing, 50])
7×5 DataFrame
│ Row │ ID    │ name   │ sex     │ height  │ weight  │
│     │ Int64 │ String │ String? │ Int64?  │ Int64?  │
├─────┼───────┼────────┼─────────┼─────────┼─────────┤
│ 1   │ 1     │ Taro   │ M       │ 170     │ 80      │
│ 2   │ 2     │ Jiro   │ missing │ 165     │ missing │
│ 3   │ 3     │ Saburo │ M       │ 175     │ missing │
│ 4   │ 4     │ Hanako │ F       │ 150     │ 40      │
│ 5   │ 5     │ Kyoko  │ F       │ missing │ 45      │
│ 6   │ 6     │ Tomoko │ missing │ 160     │ missing │
│ 7   │ 7     │ Akemi  │ F       │ missing │ 50      │

(@v1.5) pkg> add Missings

# Sexのデータで欠損値を含むレコードを削除
julia> dropmissing(df, :sex)
5×5 DataFrame
│ Row │ ID    │ name   │ sex    │ height  │ weight  │
│     │ Int64 │ String │ String │ Int64?  │ Int64?  │
├─────┼───────┼────────┼────────┼─────────┼─────────┤
│ 1   │ 1     │ Taro   │ M      │ 170     │ 80      │
│ 2   │ 3     │ Saburo │ M      │ 175     │ missing │
│ 3   │ 4     │ Hanako │ F      │ 150     │ 40      │
│ 4   │ 5     │ Kyoko  │ F      │ missing │ 45      │
│ 5   │ 7     │ Akemi  │ F      │ missing │ 50      │

# 欠損値を含むレコードを全て削除
julia> dropmissing(df, disallowmissing=true)
2×5 DataFrame
│ Row │ ID    │ name   │ sex    │ height │ weight │
│     │ Int64 │ String │ String │ Int64  │ Int64  │
├─────┼───────┼────────┼────────┼────────┼────────┤
│ 1   │ 1     │ Taro   │ M      │ 170    │ 80     │
│ 2   │ 4     │ Hanako │ F      │ 150    │ 40     │


# 統計パッケージ
julia> df = DataFrame(ID = 1:7,
name = ["Taro", "Jiro", "Saburo", "Hanako", "Kyoko", "Tomoko", "Akemi"],                                          
sex = ["M", missing, "M", "F", "F", missing, "F"],      
height = [170, 165, 175, 150, missing, 160, missing],    
weight = [80, missing, missing, 40, 45, missing, 50])
7×5 DataFrame
│ Row │ ID    │ name   │ sex     │ height  │ weight  │
│     │ Int64 │ String │ String? │ Int64?  │ Int64?  │
├─────┼───────┼────────┼─────────┼─────────┼─────────┤
│ 1   │ 1     │ Taro   │ M       │ 170     │ 80      │
│ 2   │ 2     │ Jiro   │ missing │ 165     │ missing │
│ 3   │ 3     │ Saburo │ M       │ 175     │ missing │
│ 4   │ 4     │ Hanako │ F       │ 150     │ 40      │
│ 5   │ 5     │ Kyoko  │ F       │ missing │ 45      │
│ 6   │ 6     │ Tomoko │ missing │ 160     │ missing │
│ 7   │ 7     │ Akemi  │ F       │ missing │ 50      │


julia> using Statistics

# 平均身長の計算。欠損値を含むので結果はmissingとなる。
julia> mean(df[:,4])
missing

# 欠損値を含むデータは、以下のように関数skipmissing()を用いて、欠損値を除いてから計算する。
julia> mean(skipmissing(df[:,4]))
164.0

# 列方向、行方向の平均値の計算例
julia> df = DataFrame(rand(5, 4)*10)
5×4 DataFrame
│ Row │ x1      │ x2       │ x3       │ x4      │
│     │ Float64 │ Float64  │ Float64  │ Float64 │
├─────┼─────────┼──────────┼──────────┼─────────┤
│ 1   │ 7.76653 │ 4.22065  │ 5.0639   │ 9.12496 │
│ 2   │ 7.91977 │ 3.68002  │ 1.78185  │ 3.7278  │
│ 3   │ 1.76752 │ 8.23027  │ 0.404553 │ 6.5148  │
│ 4   │ 1.52218 │ 4.01894  │ 2.07256  │ 3.84892 │
│ 5   │ 8.29163 │ 0.257238 │ 9.08174  │ 4.24294 │

# データフレームから配列への変換
julia> df1 = Array(df)
5×4 Array{Float64,2}:
7.76653  4.22065   5.0639    9.12496
7.91977  3.68002   1.78185   3.7278
1.76752  8.23027   0.404553  6.5148
1.52218  4.01894   2.07256   3.84892
8.29163  0.257238  9.08174   4.24294

# 列方向の平均の計算
julia> mean(df1, dims=1)
1×4 Array{Float64,2}:
5.45352  4.08142  3.68092  5.49188

# 行方向の平均の計算
julia> mean(df1, dims=2)
5×1Array{Float64,2}:
6.5440089212000725
4.277362324503873
4.229283176587188
2.86564827162589
5.468386996754463

# Pythonの呼び出しと統計解析

(@v1.5)j pkg> add PyCall

julia> using PyCall

# Pythonのバイナリへのパスを設定します。
julia> ENV[“Python”] = “C:¥Users¥<User_name>¥AppData¥Local¥Programs¥Python¥python”

julia> Pkg.build("PyCall")

Julia> py"print('Hello World.')"
Hello World

julia> py"print([1, 4, 9, 16, 25])"
[1, 4, 9, 16, 25]

# Julia$をつけるとpy"＜コマンドなど＞"で使用できます。
julia> x = 10; y = 20;  
julia> py"5*$x + 6*$y" 
170

# 組み込み関数を使用したいとき、例えば、関数abs（）を使用するときは、以下のように、関数pyimport（）を用いて、mathモジュールをインポートします。これを以下のようにして、mathモジュールのメソッドが使用できます。

julia> const pyabs = py"abs"
julia> pyabs(-5.0)
5.0

# 内部モジュールを使用するとき、例えば、mathモジュールを使用するときは、以下のように、関数pyimport（）を用いて、mathモジュールをインポートします。これを以下のようにして、mathモジュールのメソッドが使用できます。

julia> const pymath = pyimport("math")
julia> pymath.sqrt([4, 9, 16, 25])

# 外部モジュールを使用するとき、例えば、Numpyを使用するときは、以下のように、関数pyimport（）を用いて、Numpyをインポートします。

julia> np = pyimport("numpy")

# そして、以下のようにNumpy のarray関数を用いて、配列を作成できます。
julia> A = np.array([[5,10,15,20,25],[20,25,30,35,40],[35,40,45,50,55],[40,45,50,55,60]])
4×5 Array{Int64,2}:
5  10  15  20  25
20  25  30  35  40
35  40  45  50  55
40  45  50  55  60

julia> A[begin:2, 1:end]
2×5 Array{Int64,2}:
5  10  15  20  25
20  25  30  35  40

julia> A[begin:1, 1:end]
1×5 Array{Int64,2}:
5  10  15  20  25

julia> x
1×5 Array{Int64,2}:
5  10  15  20  25

# また、Numpy のstatisticsモジュールを用いた統計計算の例を以下に示します。
julia> stat = pyimport("statistics")

julia> stat.mean([1,3,5,7,9])
5

julia> stat.stdev([1,3,5,7,9])
3.1622776601683795

# Rの呼び出しと統計解析
(@v1.5)j pkg> add RCall

RCallのインストールが終わったら、以下のようにしてインポートします。
julia> using Rcall

# Juliaから呼び出すRは、システムのパスに設定されているものが自動的に

# 以下のように、RのコマンドをR"…"で囲むと、RのコマンドがのREPLから実行できる。
julia> R"1 + 1"
RObject{RealSxp}[1] 
2

# R側の評価値を取り出すには、rcopyを用いる。
julia> rcopy(R"1 + 1")
2.0

julia> rcopy(R"1:10")
10-element Array{Int64,1}:
1
2
3
4
5
6
7
8
9
10

# R側にJuliaの変数の値などを渡すには$で変数を埋め込む。

julia> x = 10

julia> R"$x + 50"
RObject{RealSxp}[1] 
60

# 次に、RDatasetsを用いてRによるデータ解析の例を示します。まず、RDatasetsモジュール
(@v1.5)j pkg> add RDatasets

RDatasetsをインポートする。
julia> using RDatasets

julia> iris = dataset("datasets", "iris");

# 以下で、irisデータセットの上から６行までを確認します。
julia> R"head($iris)"
RObject{VecSxp}SepalLength SepalWidth PetalLength PetalWidth Species
1         5.1        3.5         1.4        0.2  setosa
2         4.9        3.0         1.4        0.2  setosa
3         4.7        3.2         1.3        0.2  setosa
4         4.6        3.1         1.5        0.2  setosa
5         5.0        3.6         1.4        0.2  setosa
6         5.4        3.9         1.7        0.4  setosa

# irisデータセットの二次元プロットを作成します。
julia> R"plot($iris)"

 
# JuliaのREPLから$キーを叩くと、RのREPLに移行することができます。Juliaに戻るにはバックスペースキーを叩きます。

R> X=c(79, 55, 67, 80, 57)

R> mean(X)
[1] 67.6

R> var(X)
[1] 138.8

R> sd(X)[1] 
11.78134