###########################
# 第４章　グラフィックス
###########################
#Juliaのグラフィックス環境は、標準パッケージとしては用意されていませんが、いくつかのパッケージが使えます。Juliaで、よく行われる視覚化方法は、以下の３つであると言われています。

#（１）Plots.jlをフロントエンドとし、Plotly .jl、GR.jl、PyPlot.jlなどをバックエンドとして視覚化する方法
#（２）PyPlotパッケージを使う方法
#（３）Gadflyパッケージを使う方法

#本章では主に、（１）の方法を中心に、Juliaのグラフィックスパッケージのうち、Plots.jl、GR.jl、PyPlot.jlなどを紹介します。
最初に以下のように必要なパッケージをインストールしておきます。

#(@v1.5) pkg> add Plots
#(@v1.5) pkg> add GR
#(@v1.5) pkg> add PyPlot
#(@v1.5) pkg> add Gadfly


#（１）Plotパッケージを用いたグラフィックス

#まず、Plotパッケージでデフォルトのパッケージを用いた描画を示します。
100点の乱数を発生させ、プロットしてみます。
julia> using Plots
julia> x = 1:100;
julia> y = rand(100, 1);
julia> plot(x, y)

julia> savefig("plot100.png")

#すると、以下のようなグラフが描けます。
 

# 100点の2群の乱数のプロットを同時に描くには以下のようにします。

julia> y = rand(100, 2)
julia> p = plot(x, y) 

julia> savefig("plot100_2.png")

 ]
# さらに、100個の乱数zを発生させ、plot!を用いて上記のpのグラフに上書きしたグラフを描画するには以下のようにします。

julia> z = rand(100);
julia> p = plot(x, y)
julia> pz = plot!(p, x, z)

#（２）グラフの修飾

# このようなグラフに、タイトルと、凡例（ラベル）を追加し、線の太さを太くするには、以下のように引数で、指定します。

julia> x = 1:100; y = rand(100, 2) # 2 columns means two lines
julia> plot(x, y, title = "Two Lines", label = ["Line 1" "Line 2"], lw = 3)

julia> savefig("twolines.png")
 

# さらにxラベルとyラベルを追加するには以下のようにします。

julia> xlabel!("My x label");ylabel!("My y label")

julia> savefig("twolines_label.png")


# また、以下の[:circle :diamond]のように、配列を用いて、指定することも可能です。

julia> x = 1:100; y = rand(100, 2)

julia> markershapes= [:circle :diamond]; 
julia> markercolors= [:red :blue]; 
julia> plot(x, y, 
title = "MyPlot", 
xlabel = "My x label", 
ylabel = "My y label", 
label = ["1", "2"], 
color = markercolors, 
shape = markershapes, 
annotation = [(2, .3, "Attention!!!")])

julia> savefig("sin.png")

# さらにに、いろいろ試してみましょう。Plotsパッケージを用いて正弦曲線（サインカーブ）に余弦曲線（コサインカーブ）を上書きしたグラフを描いてみます。

julia> using Plots
julia> plot(sin)
julia> plot!(cos) #グラフを上書きします。

julia> savefig("sincos.png")

# その結果、以下のような図を描画できます。


julia> x = 1:100; y = rand(100, 2) # 2 columns means two lines

# プロットの軸ラベル、曲線、凡例などの修飾の例を以下に示します。

plot(sin,               #正弦曲線（サインカーブ）を描画
xlabel       ="X_value",   #X軸のラベル
ylabel       ="Y_value",   #Y軸のラベル
label        ="Sin_Curve ",     #凡例のテキスト
xlims        =(-4,4),      #X軸の範囲
ylims        =(-4,4),      #Y軸の範囲
aspect_ratio =0.5,         #表示のY/Xのアスペクト比
title        ="Sin Curve",     #タイトル
linecolor    =:green,       #線の色
linewidth    =3,           #線幅
linestyle    =:dot,        #線種
size         =(400,300),   #プロットのサイズ)
)

julia> savefig("sin.png")

# これで、以下のような図を描画できます。


# （３）様々なグラフ

#いろいろな種類のグラフを書きたいとき、seriestype = "..."で指定します。と問えば、散布図を描きたい場合は、以下のように、seriestype = :scatterで、指定します。stでも指定できます。執筆時点のマニュアルによると、以下のような種類のグラフが描画できると紹介されています。

#:none, :line, :path, :steppre, :steppost, :sticks, :scatter, :heatmap, :hexbin, :barbins, :barhist, :histogram, :scatterbins, :scatterhist, :stepbins, :stephist, :bins2d, :histogram2d, :histogram3d, :density, :bar, :hline, :vline, :contour, :pie, :shape, :image, :path3d, :scatter3d, :surface, :wireframe, :contour3d, :volume, :mesh3d

# 以下にいくつかの描画例を示します。

#（3）-1. 散布図

julia> x = rand(100);y = rand(100);
julia> plot(x, y, seriestype = :scatter, legend = false, color = [:red])

julia> savefig("scatter.png")

 

#（3）-2. 棒グラフ

#棒グラフの記入例を以下に示します。
julia> x = 1:10;
julia> y = exp.(x);
julia> plot(x, y, st = :bar, legend = false)

julia> savefig("bar.png")

 

#（3）-3. ヒストグラム

julia> x = randn(1000);
julia> plot(x, seriestype = :histogram, legend = false)

julia> savefig("histgram.png")

 

#（4）様々なバックエンド
#Juliaのグラフィックスは、さまざまなグラフィックスパッケージ(GR.jl、PyPlot.jlなど)をバックエンドにして、いろいろな美しいグラフが描けます。バックエンドのパッケージごとに依存性のあるソフトの要件が異なるので、調べてインストールするとよいでしょう。紙面の関係で、以下にバックエンドの簡単な呼び出し例のみ示します。なお、デフォルトでは、GRパッケージがバックエンドになっています。

#また、using PyPlotなどとして、本節でバックエンドで使用するグラフィックスパッケージを、Plotsモジュールを介さず、直接呼び出して操作することも可能です。これについての紹介は、紙面の関係でここでは取り上げません。他の書籍（「１から始めるJuliaプログラミング」（コロナ社））や、各パッケージのマニュアルなどを参照してください。

#（4）-1. Plotlyをバックエンドにしたグラフ
#インストール方法：
最初にpythonでplotlyをインストールしてから、Pkg.add("Plotly")を実施する。
$ python3 -m pip install plotly
$ Julia
$ ]
(@v1.5) pkg> add Plotly

使用例：
julia> using Plots
julia> plotly() # Set the backend to Plotly
julia> plot((1:100), (rand(100, 2)), title = "This is Plotted using Plotly")

julia> png("ployly.png")

 

#（4）-2. GRをバックエンドにしたグラフ

#インストール方法：
$ ]
(@v1.5) pkg> add GR

#使用例：
julia> using Plots
julia> gr() # Set the backend to gr(
julia> plot((1:100), (rand(100, 2)), st=:scatter, title = "This is Plotted using GR")
julia> png("gr.png")
 

#（4）-3. PyPlotをバックエンドにしたグラフ
#インストール方法：
最初にpythonでmatplotlibをインストールしてから、Pkg.add("PyPlot ")を実施する。
$ python3 -m pip install matplotlib
$ julia
$ ]
(@v1.5) pkg> add PyPlot

# 3 使用例：
julia> using Plots
julia> pyplot()
julia> plot((1:30), (rand(30, 2)), title = "This is Plotted using PyPlot")

julia> png("pyplot.png")
 

# （5）複数グラフの描画
# ５つのグラフを縦に配置する例を以下に示します。

julia> using Plots
julia> x = 1:10
julia> y = rand(10, 5)
julia> plot(x, y, layout = (5, 1))

julia> png("five_graph.png")


# さらに、４つのグラフを縦横に配置する例を以下に示します。

plot(scatter(x, y), 
plot(x, y), 
histogram(x, y), 
plot(x, y, lw = 3, title = "Title"), 
layout = (2, 2), legend = false)

julia> png("four_graph.png")

 

#（6）PyPlotパッケージを用いた描画
# ここでは、Plots()関数を用いて、PyPlotパッケージをバックエンドとするこれまでのやり方ではなく、using PyPlotでPyPlotを呼び出し、直接PyPlotで描画する方法を示す。
# この場合、PyPlotパッケージはMatplotlibの機能のほとんどがそのまま使え、関数などもMatplotlibと共通しているところが多いので、Matplotlibを使ったことがある人には簡単に使えるようになる。詳細な使い方は、PyPlot.jlやMatplotlibのマニュアルを参照するとよい。
なお、PyPlotパッケージを使用する場合は、すでに示したようにPyPlotをインストールする前に、予めMatplotlibをインストールし、パスを通しておく必要がある。

# インストール方法：
# 最初にpythonでmatplotlibをインストールしてから、Pkg.add("PyPlot ")を実施する。
$ python3 -m pip install matplotlib
$ julia
$ ]
(@v1.5) pkg> add PyPlot

# PyPlotパッケージのロード
julia> using PyPlot

# ヒストグラムの表示
julia> hist(randn(500));hist(randn(200));hist(randn(100))
julia> png("pyplot_hist.png")

 
# 散布図の表示
julia> scatter(rand(100), rand(100), rand(100))
julia> savefig("pyplot_scatter.png")
 
# （7）Gadflyパッケージを用いた描画
# Gadfly.jlパッケージは、Plotsパッケージのバックエンドとして使われることはありませんが、Rのggplot2 をベースに Julia で実装されているプロットと可視化のライブラリで非常に美しいグラフィックスが描けるので人気があります。Rのggplot2と記述方法が似ていることから、Rのggplot2を使ったことがある人には簡単に使えるようになります。詳細な使い方は、Gadfly.jlやRのggplot2のマニュアルを参照するとよい。

# インストール方法：
$ julia
$ ]
(@v1.5) pkg> add Gadfly
# 以下は PNG 出力時に必要
$ ]
(@v1.5) pkg> add Cairo
(@v1.5) pkg> add Fontconfig
(@v1.5) pkg> add Cairo

# RDatasetsのirisデータセットを使用して、散布図を作成する
julia> using Gadfly, RDatasets
julia> plot(dataset("datasets", "iris"), x=:SepalLength, y=:SepalWidth, Geom.point)
julia> import Cairo; PNG("gadfly.png")

# RDatasetsのirisデータセットを使用して、散布図を作成する
ulia> plot(dataset("datasets", "iris"), x=:SepalLength, Geom.histogram)
julia> import Cairo; PNG("gadfly2.png")
 
# まとめ
# 本章では、グラフィックスの簡単な入門として、JuliaのグラフィックパッケージPlotsパッケージの使い方を紹介した。さらに、Plots.jlパッケージをフロントエンドに、plotly.jl、GR.jl、PyPlot.jlなどの各種グラフィックパッケージをバックエンドにした描画法を示した。さらに、PyPlot.jlやGadfly.jlの使用法も紹介した。

# 参考サイト：
# Plots.jl:
# https://github.com/JuliaPlots/Plots.jl
# http://docs.juliaplots.org/latest/

# plotly.jl:
# https://github.com/plotly/Plotly.jl
# https://plotly.com/julia/getting-started/

# GR.jl:
# https://gr-framework.org/julia.html
# https://github.com/jheinen/GR.jl

# PyPlot.jl:
# https://github.com/JuliaPy/PyPlot.jl

# Gadfly.jl:
# http://gadflyjl.org/stable/

# Matplotlib:
# https://matplotlib.org/

# ggplot2:
# https://ggplot2.tidyverse.org/
 
