################################
# 第5章　機械学習・深層学習
###############################


# Knet.jlの使用例

$ Julia
$ }
(@v1.5) pkg> add Knet


# 1) 必要な設定とパッケージのロード
ENV["COLUMNS"]=72
using Knet, MLDatasets, IterTools

# 2) 畳み込み層の定義
struct Conv; w; b; f; end
(c::Conv)(x) = c.f.(pool(conv4(c.w, x) .+ c.b))
Conv(w1,w2,cx,cy,f=relu) = Conv(param(w1,w2,cx,cy), param0(1,1,cy,1), f);

# 3) 緻密層の定義
struct Dense; w; b; f; end
(d::Dense)(x) = d.f.(d.w * mat(x) .+ d.b)
Dense(i::Int,o::Int,f=relu) = Dense(param(o,i), param0(o), f);

# 4)各層の連結様式Chainの定義
struct Chain; layers; Chain(args...)=new(args); end
(c::Chain)(x) = (for l in c.layers; x = l(x); end; x)
(c::Chain)(x,y) = nll(c(x),y)

# 5) MNIST データの取り込み
xtrn,ytrn = MNIST.traindata(Float32)
xtst,ytst = MNIST.testdata(Float32)
dtrn = minibatch(xtrn, ytrn, 100; xsize=(size(xtrn,1),size(xtrn,2),1,:))
dtst = minibatch(xtst, ytst, 100; xsize=(size(xtst,1),size(xtst,2),1,:));

# 6) 今回作成したニューラルネットワークLeNet の訓練と検証(１GPUにつき30秒ほどの時間を必要とし、最終的に99％の精度となる)
LeNet = Chain(Conv(5,5,1,20), Conv(5,5,20,50), Dense(800,500), Dense(500,10,identity))
progress!(adam(LeNet, ncycle(dtrn,10)))
accuracy(LeNet, dtst)

# 出力結果：
0.9893569844789357


xtrn,ytrn = MNIST.traindata(Float32)
xtst,ytst = MNIST.testdata(Float32)

julia> println.(summary.((xtrn,ytrn,xtst,ytst)));
28×28×60000 Array{Float32,3}
60000-element Array{Int64,1}
28×28×10000 Array{Float32,3}
10000-element Array{Int64,1}

[MNIST.convert2image(xtst[:,:,i]) for i=1:5]
 	 	 	 	 
julia> println(ytst[1:5]);
[7, 2, 1, 0, 4]

dtrn = minibatch(xtrn,ytrn,100)
dtst = minibatch(xtst,ytst,100)

import Pkg; Pkg.add("ImageIO")
import Pkg; Pkg.add("QuartzImageIO")
import Pkg; Pkg.add("ImageMagick")

# ScikitLearn.jlの使用例


julia> Import Pkg; Pkg.add RDatasets

# ScikitLearn.jlのインストール方法：
$ Julia
$ }
(@v1.5) pkg> add ScikitLearn

julia> ENV["PYTHON"]="/usr/local/bin/python3";
(v1.1) pkg> build PyCall

# １）irisデータの準備
julia> using RDatasets: dataset

# irisデータセットの取り込み
julia> iris = dataset("datasets", "iris")

# irisデータセットの確認
julia> first(iris, 5)
5×5 DataFrame. Omitted printing of 2 columns
│ Row │ SepalLength │ SepalWidth │ PetalLength │
│     │ Float64     │ Float64    │ Float64     │
├─────┼─────────────┼────────────┼─────────────┤
│ 1   │ 5.1         │ 3.5        │ 1.4         │
│ 2   │ 4.9         │ 3.0        │ 1.4         │
│ 3   │ 4.7         │ 3.2        │ 1.3         │
│ 4   │ 4.6         │ 3.1        │ 1.5         │
│ 5   │ 5.0         │ 3.6        │ 1.4         │

# irisデータセットの入力データにあたる植物のサイズ情報
# （:SepalLength, :SepalWidth, :PetalLength, :PetalWidth]）をXに、データを配列に変換した後に代入します。

julia> X = convert(Array, iris[[:SepalLength, :SepalWidth, :PetalLength, :PetalWidth]])
150×4 Array{Float64,2}:
 5.1  3.5  1.4  0.2
 4.9  3.0  1.4  0.2
 4.7  3.2  1.3  0.2
 4.6  3.1  1.5  0.2
 5.0  3.6  1.4  0.2
 5.4  3.9  1.7  0.4
 4.6  3.4  1.4  0.3
 5.0  3.4  1.5  0.2
 4.4  2.9  1.4  0.2
 4.9  3.1  1.5  0.1
 ⋮
（続く）

# irisデータセットの出力データに当たる植物の品種の情報をyに、データを配列に変換した後に代入します。
julia> y = convert(Array, iris[:Species])
150-element Array{String,1}:
 "setosa"
 "setosa"
 "setosa"
 "setosa"
 "setosa"
 "setosa"
 "setosa"
 "setosa"
 "setosa"
 "setosa"
 ⋮
（続く）

# このデータを用いた条件では、計算結果が収束せずエラーとなりますので、最初の125エントリ分だけのデータを使うことにします。
julia> X2 = copy(X)
julia> y2 = copy(y)
julia> X = (X2[1:125,:])
julia> y = (y2[1:125])
ScikitLearnパッケージをロードします。

# julia> using ScikitLearn 

# このモデルを使用するにはscikit-learnをインストールする必要があります。 以下を参照してください。
# http://scikitlearnjl.readthedocs.io/en/latest/models/#installation

julia> @sk_import linear_model: LogisticRegression

# モデルを作成します。ここではロジスティック回帰モデルを用います。

julia> model = LogisticRegression(fit_intercept=true)

# irisをモデルに当てはめます。

julia> fit!(model, X, y)

# モデルの精度を確認します。
julia> accuracy = sum(predict(model, X) .== y) / length(y)
julia> println("accuracy: $accuracy")

# 出力結果：
accuracy: 0.976


