
##########################
# 第６章　その他の補足
##########################

#（1）スクリプトの実行とファイルの入出力

#本章では、Juliaで書かれたスクリプトを実行したり、ファイルの出入力を行ったりするプロセスを紹介します。

#（1）−１スクリプトを用いたファイルの読み書き
# 以下のようなコマンド群を順次実行してみます。

a = parse(Float64,readline(stdin))
b = parse(Float64, readline(stdin)) 
println(stdout, "Got values: $a, $b")
println(stdout, a, "+", b, "=", a+b)

# 以下をタイプすると、何か数字を入力するように、プロンプトが待機状態になりますので、３とタイプしてて[Enter]キー（または、[Return]キー）を押します。
julia> a = parse(Float64, readline(stdin))
3
3.0

# 以下をタイプすると、何か数字を入力するように、プロンプトが待機状態になりますので、３とタイプしてて[Enter]キー（または、[Return]キー）を押します。
julia> b = parse(Float64,　readline(stdin))
5
5.0

# それぞれ、a、 bに3.0, 5.0が入力されたことが、以下で確認できます。
julia> println(stdout, "Got　values: $a, $b")
Got values: 3.0, 5.0

# 演算結果が表示されます。
julia> println(stdout, a, "+", b, "=", a+b)
3.0+5.0=8.0

# 次に、先ほどの以下のコマンド群を記述したテキストファイルを作成し、add.jlという名前をつけて保存します。このようなコマンドを並べた短いドキュメントをスクリプトと言います。俗に言うプログラム、あるいはコードです。

a = parse(Float64, readline(stdin))
b = parse(Float64, readline(stdin))
println(stdout, "Got values: $a, $b")
println(stdout, a, "+", b, "=", a+b)

# その後、Windowsのコマンドプロンプト(wslの端末を使ったほうが望ましいですが、)やmacOSの端末で、julia add.jlをタイプし、[Enter]キー（または、[Return]キー）を押すと、プロンプトが、入力待ち状態になりますので、3をタイプし、[Enter]キー（または、[Return]キー）を押し、5をタイプし、[Enter]キー（または、[Return]キー）を押すと、以下のようなスクリプトの実行結果が表示されます。

% julia add.jl
3
5
Got values: 3.0, 5.0
3.0+5.0=8.0

次に、テキストエディタで（Windowsでいうメモ帳）、以下の入力数字をタイプして、input.txtと保存します。
3
5

# 以下のようにこのコマンドに「> input.txt」をつけて実行すると、input.txtのデータが読み込まれ、出力結果が表示されます。

% julia add.jl < input.txt
Got values: 3.0, 5.0
3.0+5.0=8.0

# さらに、このコマンドに「> output.txt」をつけて実行すると、出力結果が、output.txtに記入されたます。
% julia add.jl < input.txt > output.txt

% cat output.txt
Got values: 3.0, 5.0
3.0+5.0=8.0

#（1）−2テキストファイルの読み書き
#（1）−１で作成したinput.txtを読み込むには以下のようにする。

julia> f = open("input.txt", "r")
IOStream(<file input.txt>)

# input.txtの１行目を書き出しするには以下のようにします。
julia> readline(f)
"3"

# input.txtの2行目を書き出しするには以下のようにします。
julia> readline(f)
"5"

# input.txtの3行目の書き出しするには以下のようにします。、２行しかないので、データはなし。
julia> readline(f)
""

# 以下のようにして、ファイルが終了していることを確認する。
julia> eof(f)
true

# オープンしたストリームのクローズは必須です。
julia> close(f)

# input.txtを読み込んで１行ずつ表示する
julia> lines = open("input.txt", "r") do f
           readlines(f)
       end
2-element Array{String,1}:
 "3"
 "5"

julia> lines
2-element Array{String,1}:
 "3"
 "5"

# Input3.txtにデータを書き込むには以下のようにする。
julia> f = open("input3.txt", "w")
IOStream(<file input3.txt>)

# input3.txtへ１、２、３の書き込み
julia> write(f, "1\n2\n3\n")
6

# オープンしたストリームのクローズは必須です。
julia> close(f)

# 書き込んだinput3.txtの内容を確認するには以下のようにします。
julia> lines = open("input3.txt", "r") do f
           readlines(f)
       end
3-element Array{String,1}:
 "1"
 "2"
 "3"

#（1）−3エクセルファイルの読み書き

# 最初に、（1）−2で作成したinput3.txtをエクセルで開き、エクセル形式（.xlsx）及びCSV形式（.csv）で保存しておく。

# XLSXパッケージのインストール：
(@v1.5) pkg> add XLSX

# XLSXパッケージのロード：
julia> using XLSX

# input3.xlsxの読み込み
julia> xf = XLSX.readxlsx("input3.xlsx")
XLSXFile("input3.xlsx") containing 1 Worksheet
            sheetname size          range        
-------------------------------------------------
               input3 3x1           A1:A3        

# シート名の表示
julia> XLSX.sheetnames(xf)
1-element Array{String,1}:
 "input3"

# シートの範囲
julia> sh = xf["input3"]
3×1 XLSX.Worksheet: 
["input3"](A1:A3)

# A1要素を表示するには以下のようにします。
julia> sh["A1"]
1

# A2要素を表示するには以下のようにします。
julia> sh["A2"]
2

# A3要素を表示するには以下のようにします。
julia> sh["A3"]
3

# A1〜A3要素を配列として表示するには以下のようにします。
julia> sh["A1:A3"]
3×1 Array{Any,2}:
 1
 2
 3

julia> XLSX.readdata("input3.xlsx", "input3", "A1:A3")
3×1 Array{Any,2}:
 1
 2
 3

# input3.xlsxをデータフレームで表示するには以下のようにします。
julia> df = DataFrame(XLSX.readtable("input3.xlsx", "input3",header=false)...)
3×1 DataFrame
│ Row │ A   │
│     │ Any │
├─────┼─────┤
│ 1   │ 1   │
│ 2   │ 2   │
│ 3   │ 3   │

# データフレームdfをoutput3.xlsxに書き込むには以下のようにします。
julia> XLSX.writetable("output3.xlsx", df)

#（1）−5  CSVファイルの読み書き

# 最初に、（1）−2で作成したinput3.txtをエクセルで開き、CSV形式（.csv）で保存しておく。

# CSVパッケージのインストール：
(@v1.5) pkg> add CSV

# パッケージのロード
julia> using CSV
julia> using DataFrames

# input3.csv の読み込み
julia> df = CSV.read("input3.csv", header=false, delim=',')
3×1 DataFrame
│ Row │ Column1 │
│     │ Int64   │
├─────┼─────────┤
│ 1   │ 1       │
│ 2   │ 2       │
│ 3   │ 3       │

# output2.csv への書き込み
julia> df |> CSV.write("output2.csv",delim=',',writeheader=false)
"output2.csv"

# output2.csv への書き込み結果の確認
julia> df = CSV.read("output2.csv", header=false, delim=',')
3×1 DataFrame
│ Row │ Column1 │
│     │ Int64   │
├─────┼─────────┤
│ 1   │ 1       │
│ 2   │ 2       │
│ 3   │ 3       │


